import React, { Component } from "react";
import axios from "axios";
import AppContext from "@context/AppContext";

export default class FrmRemark extends Component {
    constructor() {
        super();

        this.state = {
            minChars: 5,
            enabledSubmit: false,
        };
    }
    static contextType = AppContext;

    onChangeRemark(e) {
        if (
            e.target.value != "" &&
            e.target.value.length >= this.state.minChars
        ) {
            this.setState({
                ...this.state,
                enabledSubmit: true,
            });
        } else {
            this.setState({
                ...this.state,
                enabledSubmit: false,
            });
        }
    }

    saveRemark() {
        try {
            const remarkInput = document.getElementById("remark");
            const data = {
                id: this.props.registry.id,
                estatus: remarkInput.value,
                observaciones: remarkInput.value,
                observacionesIP: remarkInput.value,
            };

            axios.post(this.props.url, data).then((response) => {
                this.context.setMessage({
                    type: "success",
                    message: response.data.message,
                });

                axios
                    .get(this.props.urlData)
                    .then((response) => {
                        this.props.updateData(response);
                    })
                    .catch((error) => {
                        console.error(error);

                        this.context.setMessage({
                            type: "error",
                            message: "Ocurrió un error, intente más tarde",
                        });
                    });

                this.props.closeModal();
            });
        } catch (error) {
            console.error(error);

            this.context.setMessage({
                type: "error",
                message: "Ocurrió un error al guardar las observaciones.",
            });
        }
    }

    componentDidMount() {
        if (
            this.props.registry.observaciones &&
            this.props.registry.observaciones.length >= this.state.minChars
        ) {
            this.setState({
                ...this.state,
                enabledSubmit: true,
            });
        }
    }

    render() {
        return (
            <div className='FrmRemark Dialog'>
                <div className='Dialog__Content'>
                    <div className='Dialog__Title'>
                        {this.props.registry.nombre ??
                            this.props.registry.nombre_cliente}{" "}
                        - observaciones
                    </div>
                    <div className='Dialog__Body'>
                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='remark'
                                    id='remark'
                                    defaultValue={
                                        this.props.registry.observaciones ?? ""
                                    }
                                    onChange={(e) => this.onChangeRemark(e)}
                                />
                                <label htmlFor='remark'>Observaciones</label>
                            </div>
                        </div>
                    </div>
                    <div className='Dialog__Footer'>
                        <div
                            className='Button Button__secondary'
                            onClick={() => this.props.closeModal()}>
                            Cerrar
                        </div>
                        <div
                            className={`Button ${
                                this.state.enabledSubmit
                                    ? "Button__primary"
                                    : "Button__Disabled"
                            }`}
                            onClick={
                                this.state.enabledSubmit
                                    ? () => this.saveRemark()
                                    : null
                            }>
                            Guardar
                        </div>
                    </div>
                </div>
            </div>
        );
    }
}
