import React, { Component } from "react";
import axios from "axios";
import Image from "next/image";

import logoAti from "@img/logo_ati.jpg";

import endpoints from "@services/Api/endpoints";
import Commons from "@utils/Commons";
import ConvertArray from "@utils/ConvertArray";
import AppContext from "@context/AppContext";
import DialogConfirm from "@components/DialogConfirm";

export default class FrmEducation extends Component {
    constructor() {
        super();

        this.state = {
            title: "Registro de educación",
            titleReview: "Revisión de la oportunidad",
            urlSave: endpoints.education.store,
            showDialogConfirm: false,
            isEditing: true,
        };
    }

    static contextType = AppContext;

    validateForm() {
        let textInputs = document.querySelectorAll(
            ".Form__Education input[type=text]"
        );

        let linkInputs = document.querySelectorAll(
            `.Form__Education input[type=url]`
        );

        let inputsToValidate = [...textInputs, ...linkInputs];
        Commons.validateInputs(inputsToValidate);
    }

    getFormData() {
        let formData = [];
        formData["usuario"] = this.context.state.user.name;

        let textInputs = document.querySelectorAll(
            ".Form__Education input[type=text]"
        );

        let linkInputs = document.querySelectorAll(
            `.Form__Education input[type=url]`
        );

        let inputs = [...textInputs, ...linkInputs];

        for (const input of inputs) {
            formData[input.name] = input.value;
        }

        return formData;
    }

    storeEducation() {
        const formData = this.getFormData();
        const data = ConvertArray.toObject(formData);

        try {
            axios
                .post(this.state.urlSave, data)
                .then((response) => {
                    this.context.setMessage({
                        type: "success",
                        message: "Educación registrada correctamente",
                    });

                    axios.get(this.props.url).then((response) => {
                        this.props.updateData(response);
                    });

                    this.props.closeModal();
                })
                .catch((error) => {
                    console.error("storeEducation", error);
                    this.context.setMessage({
                        type: "error",
                        message:
                            "Ocurrió un error en el registro de la Educación",
                    });
                });
        } catch (error) {
            this.context.setMessage({
                type: "error",
                message: `Ocurrió un error al guardar la Educación: ${error.name} ${error.message}`,
            });
        }
    }

    reviewData() {
        this.validateForm();
        let submit = true;
        const inputs = document.querySelectorAll(".Form__Education input");

        for (const input of inputs) {
            if (input.parentNode.classList.contains("Invalid")) {
                this.context.setMessage({
                    type: "error",
                    message: "LLena los campos requeridos",
                });
                submit = false;
                break;
            }
        }

        if (!submit) {
            return;
        }

        this.setState({ isEditing: false }, () => {
            this.context.setMessage({
                type: "info",
                message:
                    "Revisa que los datos de la nueva oportunidad sean correctos, de lo contrario has las modificaciones correspondienes",
            });
        });
    }

    componentDidMount() {
        const inputs = document.querySelectorAll(`.Form__Input input`);

        for (const input of inputs) {
            input.addEventListener("change", (e) => {
                if (e.target.value != "") {
                    e.target.parentNode.classList.remove("Invalid");
                }
            });
        }
    }

    render() {
        return (
            <div className='FrmEducation Dialog'>
                <div className='Dialog__Content'>
                    <div className='Dialog__Title'>
                        {this.state.isEditing
                            ? this.state.title
                            : this.state.titleReview}
                    </div>
                    <div className='Dialog__Body Form__Education'>
                        {this.state.isEditing ? (
                            <div className='Logo'>
                                <Image src={logoAti} alt='Logo' />
                            </div>
                        ) : null}

                        <div className='Form__Group Col-2'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='customer_name'
                                    id='customer_name'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='customer_name'>
                                    Nombre del cliente
                                </label>
                            </div>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='contact_name'
                                    id='contact_name'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='contact_name'>
                                    Nombre del vendedor o persona de contacto
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='institution_name'
                                    id='institution_name'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='institution_name'>
                                    Nombre de la institución educativa
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='institution_description'
                                    id='institution_description'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='institution_description'>
                                    Descripción general de la institución
                                    educativa (nivel educativo, tamaño, Aprox)
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='challenge'
                                    id='challenge'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='challenge'>
                                    Cuál es el principal desafío o problema que
                                    la institución educativa enfrenta y que
                                    desea resolver con nuestra solución
                                    tecnológica?
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='goals'
                                    id='goals'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='goals'>
                                    ¿Cuáles son los objetivos educativos y
                                    estratégicos de la institución relacionados
                                    con esta solución?
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='budget'
                                    id='budget'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='budget'>
                                    ¿Cuál es el presupuesto estimado para esta
                                    solución?
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='infrastructure'
                                    id='infrastructure'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='infrastructure'>
                                    ¿Cuenta con la infraestructura adecuada para
                                    su solucion? (redes, servidores,
                                    dispositivos, software utilizado, etc.)
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='integrations'
                                    id='integrations'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='integrations'>
                                    ¿Que tecnologias tienen destinadas a las
                                    aulas actualmente?
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='requirements'
                                    id='requirements'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='requirements'>
                                    Necesidad principal?
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='criterias'
                                    id='criterias'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='criterias'>
                                    ¿Cuáles son los criterios de evaluación y
                                    selección que la institución educativa
                                    utilizará para tomar una decisión final?
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='solutions_providers'
                                    id='solutions_providers'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='solutions_providers'>
                                    ¿La institución educativa ha considerado
                                    otras soluciones o proveedores? Si es así,
                                    ¿cuáles?
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='additional_information'
                                    id='additional_information'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='additional_information'>
                                    ¿Hay alguna otra información adicional que
                                    consideres relevante para el desarrollo de
                                    la solución tecnológica en el entorno
                                    educativo?
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='url'
                                    name='link_folder'
                                    id='link_folder'
                                    required
                                    readOnly={!this.state.isEditing}
                                    placeholder='https://example.example'
                                />
                                <label htmlFor='link_folder'>
                                    Link carpeta
                                </label>
                            </div>
                        </div>
                    </div>

                    <div className='Dialog__Footer'>
                        <div
                            className='Button Button__secondary'
                            onClick={
                                this.state.isEditing
                                    ? () => this.props.closeModal()
                                    : () => this.setState({ isEditing: true })
                            }>
                            {this.state.isEditing ? "Cerrar" : "Atrás"}
                        </div>
                        {this.state.isEditing ? (
                            <div
                                className='Button Button__primary'
                                onClick={() => this.reviewData()}>
                                Continuar
                            </div>
                        ) : (
                            <div
                                className='Button Button__primary'
                                onClick={() =>
                                    this.setState({ showDialogConfirm: true })
                                }>
                                Enviar datos
                            </div>
                        )}
                    </div>
                </div>

                {this.state.showDialogConfirm ? (
                    <DialogConfirm
                        proceed={() => this.storeEducation()}
                        closeModal={() =>
                            this.setState({ showDialogConfirm: false })
                        }
                    />
                ) : null}
            </div>
        );
    }
}
