import React, { useContext, useState } from "react";
import axios from "axios";
import AppContext from "@context/AppContext";

const FrmNotification = ({ title, url, closeModal, telegramLink = null }) => {
    const { setMessage } = useContext(AppContext);

    const regexEmail = /^[\w-\.]+@([\w-]+\.)+[\w-]{2,4}$/g;
    const [valid, setValid] = useState(true);

    const saveEmail = () => {
        const email = document.getElementById("notification_email");

        if (email.value.length == "") {
            setValid(false);
            return;
        }

        axios
            .post(url, { correo: email.value })
            .then((response) => {
                setMessage({
                    type: "success",
                    message: response.data,
                });

                email.value = "";
                closeModal();
            })
            .catch((error) => {
                setMessage({
                    type: "error",
                    message: error.response.data,
                });
            });
    };

    return (
        <div className='FrmNotification Dialog'>
            <div className='Dialog__Content'>
                <div className='Dialog__Title'>{title}</div>
                <div className='Dialog__Body'>
                    {telegramLink && (
                        <div className='telegramLink'>
                            Link Telegram:{" "}
                            <a href={telegramLink}>{telegramLink}</a>
                        </div>
                    )}
                    <div className='Form__Input'>
                        <input
                            type='email'
                            name='notification_email'
                            id='notification_email'
                            required
                            onChange={(e) => {
                                if (
                                    e.target.value.length > 0 &&
                                    regexEmail.test(e.target.value)
                                ) {
                                    setValid(true);
                                } else {
                                    setValid(false);
                                }
                            }}
                        />
                        <label htmlFor='notification_email'>Correo</label>
                        {!valid && (
                            <p className='Message__Danger'>
                                El correo es obligatorio
                            </p>
                        )}
                    </div>
                </div>
                <div className='Dialog__Footer'>
                    <div
                        className='Button Button__secondary'
                        onClick={() => closeModal()}>
                        Cancelar
                    </div>
                    <div
                        className={`Button ${
                            valid ? "Button__primary" : "Button__Disabled"
                        }`}
                        onClick={() => saveEmail()}>
                        Guardar
                    </div>
                </div>
            </div>
        </div>
    );
};

export default FrmNotification;
