import React, { Component } from "react";

import logoAti from "@img/logo_ati.jpg";
import Image from "next/image";
import AppContext from "@context/AppContext";
import Commons from "@utils/Commons";
import ConvertArray from "@utils/ConvertArray";
import endpoints from "@services/Api/endpoints";
import axios from "axios";
import DialogConfirm from "./DialogConfirm";

export default class FrmPresale extends Component {
    constructor() {
        super();

        this.state = {
            title: "Registro de IP Preventa",
            titleReview: "Revisión de la oportunidad",
            showTxtBrands: false,
            urlSave: endpoints.preventa.store,
            showDialogConfirm: false,
            isEditing: true,
        };
    }

    static contextType = AppContext;

    validateForm() {
        let textInputs = document.querySelectorAll(
            ".FrmPresale input[type=text]"
        );
        let phoneInputs = document.querySelectorAll(
            ".FrmPresale input[type=tel]"
        );
        let emailInputs = document.querySelectorAll(
            ".FrmPresale input[type=email]"
        );

        let linkInputs = document.querySelectorAll(
            `.FrmPresale input[type=url]`
        );

        let inputsToValidate = [
            ...textInputs,
            ...phoneInputs,
            ...emailInputs,
            ...linkInputs,
        ];
        Commons.validateInputs(inputsToValidate);
    }

    getFormData() {
        let formData = [];
        formData["usuario"] = this.context.state.user.name;

        let textInputs = document.querySelectorAll(
            ".Form__Presale input[type=text]"
        );
        let phoneInputs = document.querySelectorAll(
            ".FrmPresale input[type=tel]"
        );
        let emailInputs = document.querySelectorAll(
            ".FrmPresale input[type=email]"
        );
        let linkInputs = document.querySelectorAll(
            `.FrmPresale input[type=url]`
        );
        let radioSolution = document.getElementsByName("solution");

        let inputs = [
            ...textInputs,
            ...phoneInputs,
            ...emailInputs,
            ...linkInputs,
        ];

        for (const input of inputs) {
            formData[input.name] = input.value;
        }

        const pushRadioValue = (radios, data) => {
            for (const radio of radios) {
                if (radio.checked) {
                    data[radio.name] = radio.value;
                }
            }

            return data;
        };

        formData = pushRadioValue(radioSolution, formData);

        return formData;
    }

    storePresale() {
        const formData = this.getFormData();
        const data = ConvertArray.toObject(formData);

        try {
            axios
                .post(this.state.urlSave, data)
                .then((response) => {
                    this.context.setMessage({
                        type: "success",
                        message: "IP Preventa registrado correctamente",
                    });

                    axios.get(this.props.url).then((response) => {
                        this.props.updateData(response);
                    });

                    this.props.closeModal();
                })
                .catch((error) => {
                    console.error("storePresale", error);
                    this.context.setMessage({
                        type: "error",
                        message:
                            "Ocurrió un error en el registro de la IP Preventa",
                    });
                });
        } catch (error) {
            this.context.setMessage({
                type: "error",
                message: `Ocurrió un error al guardar la preventa: ${error.name} ${error.message}`,
            });
        }
    }

    reviewData() {
        this.validateForm();
        let submit = true;
        const inputs = document.querySelectorAll(".FrmPresale input");

        for (const input of inputs) {
            if (input.parentNode.classList.contains("Invalid")) {
                this.context.setMessage({
                    type: "error",
                    message: "LLena los campos requeridos",
                });
                submit = false;
                break;
            }
        }

        if (!submit) {
            return;
        }

        this.setState({ isEditing: false }, () => {
            this.context.setMessage({
                type: "info",
                message:
                    "Revisa que los datos de la nueva oportunidad sean correctos, de lo contrario has las modificaciones correspondienes",
            });
        });
    }

    componentDidMount() {
        const inputs = document.querySelectorAll(`.Form__Input input`);

        for (const input of inputs) {
            input.addEventListener("change", (e) => {
                if (e.target.value != "") {
                    e.target.parentNode.classList.remove("Invalid");
                }
            });
        }
    }

    render() {
        return (
            <div className='FrmPresale Dialog'>
                <div className='Dialog__Content'>
                    <div className='Dialog__Title'>
                        {this.state.isEditing
                            ? this.state.title
                            : this.state.titleReview}
                    </div>
                    <div className='Dialog__Body Form__Presale'>
                        {this.state.isEditing ? (
                            <div className='Logo'>
                                <Image src={logoAti} alt='Logo' />
                            </div>
                        ) : null}

                        <div className='Form__Group Col-2'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='nameClient'
                                    id='nameClient'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='nameClient'>
                                    Nombre del cliente
                                </label>
                            </div>

                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='job'
                                    id='job'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='job'>Puesto</label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='decisions'
                                    id='decisions'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='decisions'>
                                    Es tomador de decisiones?
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group Col-2'>
                            <div className='Form__Input'>
                                <input
                                    type='tel'
                                    name='phone'
                                    id='phone'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='phone'>Teléfono</label>
                            </div>
                            <div className='Form__Input'>
                                <input
                                    type='email'
                                    name='mail'
                                    id='mail'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='mail'>Correo</label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='companyName'
                                    id='companyName'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='companyName'>
                                    Nombre de la empresa
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='address'
                                    id='address'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='address'>Direccion</label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='bussinesDescription'
                                    id='bussinesDescription'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='bussinesDescription'>
                                    Descripción general del negocio y la
                                    industria
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='problem'
                                    id='problem'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='problem'>
                                    Necesidad principal?
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group Col-2'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='budgetIP'
                                    id='budgetIP'
                                    min={0}
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='budgetIP'>
                                    Cual es su presupuesto
                                </label>
                            </div>

                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='solution'
                                    id='solution'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='solution'>
                                    Contra que se compite (Marcas, Solucion)
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='brands'
                                    id='brands'
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='brands'>
                                    Si la respuesta anterior es afirmativa, con
                                    que marca o marcas?
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='brandPreference'
                                    id='brandPreference'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='brandPreference'>
                                    Tiene alguna preferencia de marcas, Cuales?
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='infrastructureIP'
                                    id='infrastructureIP'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='infrastructureIP'>
                                    Cuenta con la infraestructura adecuada para
                                    su solucion
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='additionalInfo'
                                    id='additionalInfo'
                                    required
                                    readOnly={!this.state.isEditing}
                                />
                                <label htmlFor='additionalInfo'>
                                    ¿Hay alguna otra información adicional que
                                    consideres relevante para el desarrollo de
                                    la solución?
                                </label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='url'
                                    name='link_folder'
                                    id='link_folder'
                                    required
                                    readOnly={!this.state.isEditing}
                                    placeholder='https://example.example'
                                />
                                <label htmlFor='link_folder'>
                                    Link carpeta
                                </label>
                            </div>
                        </div>
                    </div>
                    <div className='Dialog__Footer'>
                        <div
                            className='Button Button__secondary'
                            onClick={
                                this.state.isEditing
                                    ? () => this.props.closeModal()
                                    : () => this.setState({ isEditing: true })
                            }>
                            {this.state.isEditing ? "Cerrar" : "Corregir"}
                        </div>

                        {this.state.isEditing ? (
                            <div
                                className='Button Button__primary'
                                onClick={() => this.reviewData()}>
                                Continuar
                            </div>
                        ) : (
                            <div
                                className='Button Button__primary'
                                onClick={() =>
                                    this.setState({ showDialogConfirm: true })
                                }>
                                Enviar datos
                            </div>
                        )}
                    </div>
                </div>
                {this.state.showDialogConfirm ? (
                    <DialogConfirm
                        proceed={() => this.storePresale()}
                        closeModal={() =>
                            this.setState({ showDialogConfirm: false })
                        }
                    />
                ) : null}
            </div>
        );
    }
}
