import axios from "axios";
import React, { Component } from "react";

import AppContext from "@context/AppContext";

import endpoints from "@services/Api/endpoints";
import Commons from "@utils/Commons";
import TelegramHelper from "@utils/TelegramHelper";
import TelegramChannels from "@utils/TelegramChannels";

export default class FrmProductSatus extends Component {
    constructor() {
        super();

        this.state = {
            statuses: Commons.msgTemplates.bid.products.statuses,
            currentStatus: "",
            showInvalidOpt: false,
        };
    }

    static contextType = AppContext;

    getTGMessage(statusKey) {
        let message = Commons.msgTemplates.bid.products.messages.filter(
            (message) => message.key === parseInt(statusKey)
        )[0];

        message = message.msg
            .replace("%BID%", this.props.registry.nombre)
            .replace("%USER%", this.context.state.user.name);

        return message;
    }

    saveStatus() {
        const selectedOpt = document.getElementById("status");
        let status = this.state.statuses.filter(
            (status) => status.value === parseInt(selectedOpt.value)
        );

        if (status.length > 0) {
            const url = endpoints.status.products.replace(
                "%ID%",
                this.props.registry.id
            );
            axios
                .post(url, { status: status[0].label })
                .then((response) => {
                    this.context.setMessage({
                        type: "success",
                        message: "Estatus actualizado correctamente",
                    });

                    axios.get(this.props.urlData).then((responseGet) => {
                        this.props.updateData(responseGet);
                        this.props.closeModal();

                        TelegramHelper.sendMessage(
                            this.getTGMessage(status[0].value),
                            TelegramChannels.bidChannel
                        );
                    });
                })
                .catch((error) => {
                    console.error(error);
                    this.context.setMessage({
                        type: "error",
                        message:
                            "Ocurrió un error al actualizar el estatus de los productos",
                    });
                });
        } else {
            this.setState({ showInvalidOpt: true });
        }
    }

    onChangeSelection(e) {
        this.setState({ currentStatus: e.target.value });
    }

    componentDidMount() {
        const status = this.state.statuses.filter(
            (status) => status.label == this.props.registry.estatus_productos
        );
        this.setState({
            currentStatus: status.length == 0 ? "" : status[0].value,
        });
    }

    componentDidUpdate() {
        if (
            this.state.currentStatus != document.getElementById("status").value
        ) {
            if (
                this.state.currentStatus != "" &&
                this.state.statuses.filter(
                    (status) =>
                        status.value === parseInt(this.state.currentStatus)
                ).length > 0
            ) {
                document.getElementById("status").value =
                    this.state.currentStatus;
            }
        }
    }

    render() {
        return (
            <div className='FrmProductSatus Dialog'>
                <div className='Dialog__Content'>
                    <div className='Dialog__Title'>
                        Actualizar estatus de productos
                    </div>
                    <div className='Dialog__Body'>
                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <select
                                    name='status'
                                    id='status'
                                    onChange={(e) => this.onChangeSelection(e)}>
                                    {this.state.statuses.map((status) => (
                                        <option
                                            value={status.value}
                                            key={status.value}>
                                            {status.label}
                                        </option>
                                    ))}
                                </select>
                                <label htmlFor='status'>Estatus</label>
                            </div>
                        </div>

                        {this.state.showInvalidOpt ? (
                            <p>El estatus seleccionado es inválido</p>
                        ) : null}
                    </div>
                    <div className='Dialog__Footer'>
                        <div
                            className='Button Button__secondary'
                            onClick={() => this.props.closeModal()}>
                            Cerrar
                        </div>
                        <div
                            className='Button Button__primary'
                            onClick={() => this.saveStatus()}>
                            Guardar
                        </div>
                    </div>
                </div>
            </div>
        );
    }
}
