import React, { Component } from "react";
import axios from "axios";

import AppContext from "@context/AppContext";

import endpoints from "@services/Api/endpoints";

import DialogConfirm from "@components/DialogConfirm";
import Commons from "@utils/Commons";
import ConvertArray from "@utils/ConvertArray";

export default class FrmRole extends Component {
    constructor() {
        super();

        this.state = {
            title: "Nuevo rol",
            showDialogConfirmDelete: false,
        };
    }

    static contextType = AppContext;

    validateForm() {
        let inputs = document.querySelectorAll(".FrmRole input");

        Commons.validateInputs(inputs);
    }

    getFormData() {
        let inputs = document.querySelectorAll(".FrmRole input");

        let formData = [];

        for (const input of inputs) {
            formData[input.name] = input.value;
        }

        return formData;
    }

    saveRole() {
        this.validateForm();
        let submit = true;

        const inputs = document.querySelectorAll(`.Form__Input input`);

        for (const input of inputs) {
            if (input.parentNode.classList.contains("Invalid")) {
                this.context.setMessage({
                    type: "error",
                    message: "LLena los campos requeridos",
                });
                submit = false;
                break;
            }
        }

        if (!submit) {
            return;
        }

        const formData = this.getFormData();
        const data = ConvertArray.toObject(formData);

        const url = this.props.role
            ? endpoints.roles.update.replace("%ID%", this.props.role.id)
            : endpoints.roles.store;

        axios
            .post(url, data)
            .then((response) => {
                this.context.setMessage({
                    type: "success",
                    message: response.data.message,
                });

                if (this.props.role == null) {
                    this.props.addRoleRow(response.data.role);
                } else {
                    this.props.updateRoleRow(response.data.role);
                }

                this.props.closeModal();
            })
            .catch((error) => {
                console.error(error);
                this.context.setMessage({
                    type: "error",
                    message: error.response.data.message,
                });
            });
    }

    deleteRole(role) {
        axios
            .delete(endpoints.roles.delete.replace("%ID%", role.id))
            .then((response) => {
                this.context.setMessage({
                    type: "success",
                    message: response.data.message,
                });

                this.props.removeRoleRow(role);
                this.props.closeModal();
            })
            .catch((error) => {
                this.context.setMessage({
                    type: "error",
                    message: error.response.data.message,
                });
            });
    }

    componentDidMount() {
        this.setState({
            title: this.props.role ? "Modificar rol" : this.state.title,
        });
    }

    render() {
        return (
            <div className='FrmRole Dialog'>
                <div className='Dialog__Content'>
                    <div className='Dialog__Title'>{this.state.title}</div>
                    <div className='Dialog__Body'>
                        <div className='Form__Group Col'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='title'
                                    id='title'
                                    required
                                    defaultValue={
                                        this.props.role
                                            ? this.props.role.title
                                            : ""
                                    }
                                />
                                <label htmlFor='title'>Nombre</label>
                            </div>
                        </div>

                        <div className='Form__Group Col'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='value'
                                    id='value'
                                    required
                                    defaultValue={
                                        this.props.role
                                            ? this.props.role.value
                                            : ""
                                    }
                                />
                                <label htmlFor='value'>Valor</label>
                            </div>
                        </div>
                    </div>
                    <div className='Dialog__Footer'>
                        <div
                            className='Button Button__secondary'
                            onClick={() => this.props.closeModal()}>
                            Cerrar
                        </div>
                        {this.props.role ? (
                            <div
                                className='Button Button__danger'
                                onClick={() =>
                                    this.setState({
                                        showDialogConfirmDelete: true,
                                    })
                                }>
                                Eliminar
                            </div>
                        ) : null}
                        <div
                            className='Button Button__primary'
                            onClick={() => this.saveRole()}>
                            Guardar
                        </div>
                    </div>
                </div>
                {this.state.showDialogConfirmDelete ? (
                    <DialogConfirm
                        description={"El rol seleccionado sera eliminado"}
                        labelContinue={"Aceptar"}
                        proceed={() => this.deleteRole(this.props.role)}
                        closeModal={() =>
                            this.setState({ showDialogConfirmDelete: false })
                        }
                    />
                ) : null}
            </div>
        );
    }
}
