import React, { Component } from "react";
import axios from "axios";

import AppContext from "@context/AppContext";
import endpoints from "@services/Api/endpoints";

import DateFormatter from "@utils/DateFormatter";
import TelegramHelper from "@utils/TelegramHelper";

export default class FrmUpdateDate extends Component {
    constructor() {
        super();

        this.state = {
            label: "",
            defaultDate: "",
        };
    }

    static contextType = AppContext;

    sendTelegramMsg() {
        const now = new Date();
        let message = `
${this.props.registry.nombre}
Fecha límite de entrega de bienes para: ${DateFormatter.dateNormal(now, true)}
        `;

        try {
            TelegramHelper.sendMessage(message);
        } catch (error) {
            console.error("Send Telegram Message", error);
        }
    }

    saveDate() {
        try {
            const date = document.getElementById("newDate").value;
            axios
                .post(endpoints.bid.deathline_date, {
                    id: this.props.registry.id,
                    fechaLBienes: date,
                })
                .then((response) => {
                    this.context.setMessage({
                        type: "success",
                        message: response.data.message,
                    });

                    this.sendTelegramMsg();

                    axios.get(this.props.url).then((response) => {
                        this.props.updateData(response);
                    });

                    this.props.closeModal();
                })
                .catch((response) => {
                    console.error(response);

                    this.context.setMessage({
                        type: "error",
                        message: "Ocurrió un error, intente más tarde",
                    });
                });
        } catch (error) {
            console.error(response);

            this.context.setMessage({
                type: "error",
                message: "Ocurrió un error, intente más tarde",
            });
        }
    }

    componentDidMount() {
        this.setState({
            label: "Fecha límite entrga de bienes",
        });

        document.getElementById("newDate").defaultValue =
            DateFormatter.dateNormal(
                this.props.registry.fecha_limite_entregab,
                true
            );
    }

    render() {
        return (
            <div className='FrmUpdateDate Dialog'>
                <div className='Dialog__Content'>
                    <div className='Dialog__Title'>
                        {this.props.registry.nombre}
                    </div>
                    <div className='Dialog__Body'>
                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='date'
                                    name='newDate'
                                    id='newDate'
                                />
                                <label htmlFor='newDate'>
                                    {this.state.label}
                                </label>
                            </div>
                        </div>
                    </div>
                    <div className='Dialog__Footer'>
                        <div
                            className='Button Button__secondart'
                            onClick={() => this.props.closeModal()}>
                            Cerrar
                        </div>
                        <div
                            className='Button Button__primary'
                            onClick={() => this.saveDate()}>
                            Guardar
                        </div>
                    </div>
                </div>
            </div>
        );
    }
}
