import React, { Component } from "react";
import axios from "axios";

import AppContext from "@context/AppContext";

import endpoints from "@services/Api/endpoints";

import Commons from "@utils/Commons";
import ConvertArray from "@utils/ConvertArray";

import DialogConfirm from "@components/DialogConfirm";
import FrmRole from "@components/FrmRole";

export default class FrmUser extends Component {
    constructor() {
        super();

        this.state = {
            title: "Nuevo usuario",
            roles: [],
            rolesValues: [],
            currentRole: 0,
            showDialogConfirm: false,
            showDialogConfirmRestore: false,
            showFrmRoles: false,
        };
    }

    static contextType = AppContext;

    validateForm() {
        let inputs = document.querySelectorAll(".FrmUser input");
        let select = document.querySelector(".FrmUser select#role");

        let inputsToValidate = [...inputs, select];

        Commons.validateInputs(inputsToValidate);
    }

    getFormData() {
        let inputs = document.querySelectorAll(".FrmUser input");
        let select = document.querySelector(".FrmUser select#role");

        let allInputs = [...inputs, select];

        let formData = [];

        for (const input of allInputs) {
            formData[input.name] = input.value;
        }

        return formData;
    }

    saveUser() {
        this.validateForm();
        let submit = true;

        const inputs = document.querySelectorAll(`.Form__Input input`);
        const select = document.querySelector(".FrmUser select#role");

        for (const input of [...inputs, select]) {
            if (input.parentNode.classList.contains("Invalid")) {
                this.context.setMessage({
                    type: "error",
                    message: "LLena los campos requeridos",
                });
                submit = false;
                break;
            }
        }

        if (!submit) {
            return;
        }

        const formData = this.getFormData();
        const data = ConvertArray.toObject(formData);

        const url = this.props.user
            ? endpoints.users.update.replace("%ID%", this.props.user.id)
            : endpoints.users.store;
        axios
            .post(url, data)
            .then((response) => {
                this.context.setMessage({
                    type: "success",
                    message: response.data.message,
                });

                if (this.props.user == null) {
                    this.props.newUserRow(response.data.user);
                } else {
                    this.props.updateUserRow(response.data.user);
                }

                this.props.closeModal();
            })
            .catch((error) => {
                console.error(error);
                this.context.setMessage({
                    type: "error",
                    message: error.response.data.message,
                });
            });
    }

    deleteUser(user) {
        axios
            .delete(endpoints.users.delete.replace("%ID%", user.id))
            .then((response) => {
                this.context.setMessage({
                    type: "success",
                    message: response.data.message,
                });

                this.props.updateUserRow(response.data.user);

                this.props.closeModal();
            })
            .catch((error) => {
                console.error(error);
                this.context.setMessage({
                    type: "error",
                    message: error.response.data.message,
                });
            });
    }

    restoreUser(user) {
        axios
            .patch(endpoints.users.restore.replace("%ID%", user.id))
            .then((response) => {
                this.context.setMessage({
                    type: "success",
                    message: response.data.message,
                });

                this.props.updateUserRow(response.data.user);

                this.props.closeModal();
            })
            .catch((error) => {
                console.error(error);
                this.context.setMessage({
                    type: "error",
                    message: error.response.data.message,
                });
            });
    }

    addRoleRow(role) {
        this.setState({
            roles: [role, ...this.state.roles],
            rolesValues: [role.id, ...this.state.rolesValues],
        });
    }

    async fetchRoles() {
        await axios
            .get(endpoints.roles.get)
            .then((response) => {
                const roles = response.data.roles;
                this.setState({
                    roles: roles,
                    rolesValues: roles.map((role) => role.id),
                });
            })
            .catch((error) => {
                console.error(error);
                this.context.setMessage({
                    type: "error",
                    message: "Ocurrio un error al obtener el listado de roles",
                });
            });
    }

    onChangeSelection(e) {
        this.setState({ currentRole: e.target.value });
    }

    componentDidMount() {
        this.setState(
            {
                title: this.props.user ? "Modificar usuario" : this.state.title,
                user: this.props.user,
                currentRole: this.props.user ? this.props.user.role.id : 0,
            },
            () => {
                this.fetchRoles();
            }
        );
    }

    componentDidUpdate() {
        if (this.state.currentRole != document.getElementById("role").value) {
            if (
                this.state.currentRole != "" &&
                this.state.rolesValues.includes(this.state.currentRole)
            ) {
                console.log("this.state.currentRole", this.state.currentRole);
                document.getElementById("role").value = this.state.currentRole;
            }
        }
    }

    render() {
        return (
            <div className='FrmUser Dialog'>
                <div className='Dialog__Content'>
                    <div className='Dialog__Title'>{this.state.title}</div>
                    <div className='Dialog__Body'>
                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='text'
                                    name='name'
                                    id='name'
                                    defaultValue={
                                        this.props.user
                                            ? this.props.user.name
                                            : ""
                                    }
                                    required
                                    disabled={
                                        this.props.user
                                            ? this.props.user.deleted
                                            : false
                                    }
                                />
                                <label htmlFor='name'>Nombre completo</label>
                            </div>
                        </div>

                        <div className='Form__Group'>
                            <div className='Form__Input'>
                                <input
                                    type='email'
                                    name='email'
                                    id='email'
                                    defaultValue={
                                        this.props.user
                                            ? this.props.user.email
                                            : ""
                                    }
                                    required
                                    disabled={
                                        this.props.user
                                            ? this.props.user.deleted
                                            : false
                                    }
                                />
                                <label htmlFor='email'>Correo</label>
                            </div>
                        </div>

                        <div className='Form__Group Col-2'>
                            <div className='Form__Input'>
                                <select
                                    name='role'
                                    id='role'
                                    required
                                    disabled={
                                        this.props.user
                                            ? this.props.user.deleted
                                            : false
                                    }
                                    onChange={(e) => this.onChangeSelection(e)}>
                                    {this.state.roles.map((role) => (
                                        <option
                                            value={role.id}
                                            key={role.value}>
                                            {role.title}
                                        </option>
                                    ))}
                                </select>
                                <label htmlFor='role'>Role de usuario</label>
                            </div>
                            {!this.props.user ||
                            this.props.user.deleted == false ? (
                                <div className='Form__Input'>
                                    <div
                                        className='Button Button__primary'
                                        onClick={() =>
                                            this.setState({
                                                showFrmRoles: true,
                                            })
                                        }>
                                        Nuevo
                                    </div>
                                </div>
                            ) : null}
                        </div>
                        {this.state.roles.length === 0 ? (
                            <small>
                                No se pueden registrar usuarios sin rol
                            </small>
                        ) : null}
                    </div>
                    <div className='Dialog__Footer'>
                        <div
                            className='Button Button__secondary'
                            onClick={() => this.props.closeModal()}>
                            Cerrar
                        </div>
                        {this.props.user ? (
                            <>
                                {this.props.user.deleted ? (
                                    <div
                                        className='Button Button__success'
                                        onClick={() =>
                                            this.setState({
                                                showDialogConfirmRestore: true,
                                            })
                                        }>
                                        Restaurar
                                    </div>
                                ) : (
                                    <div
                                        className='Button Button__danger'
                                        onClick={() =>
                                            this.setState({
                                                showDialogConfirm: true,
                                            })
                                        }>
                                        Eliminar
                                    </div>
                                )}
                            </>
                        ) : null}
                        {this.props.user && this.props.user.deleted ? null : (
                            <div
                                className={`Button Button__primary ${
                                    this.state.roles.length == 0
                                        ? "Disabled"
                                        : ""
                                }`}
                                onClick={
                                    this.state.roles.length == 0
                                        ? null
                                        : () => this.saveUser()
                                }>
                                Guardar
                            </div>
                        )}
                    </div>
                </div>

                {this.state.showDialogConfirm ? (
                    <DialogConfirm
                        description={
                            "El usuario seleccionado sera liminado, ¿desea continuar?"
                        }
                        labelContinue={"Aceptar"}
                        proceed={() => this.deleteUser(this.props.user)}
                        closeModal={() =>
                            this.setState({ showDialogConfirm: false })
                        }
                    />
                ) : null}

                {this.state.showDialogConfirmRestore ? (
                    <DialogConfirm
                        description={
                            "El usuario seleccionado sera restaurado y podra acceder a la aplicacion con normalidad"
                        }
                        labelContinue={"Aceptar"}
                        proceed={() => this.restoreUser(this.props.user)}
                        closeModal={() =>
                            this.setState({ showDialogConfirmRestore: false })
                        }
                    />
                ) : null}

                {this.state.showFrmRoles ? (
                    <FrmRole
                        role={null}
                        addRoleRow={(role) => this.addRoleRow(role)}
                        closeModal={() =>
                            this.setState({ showFrmRoles: false })
                        }
                    />
                ) : null}
            </div>
        );
    }
}
