import React, { useEffect, useState } from "react";
import Image from "next/image";
import { useRouter } from "next/navigation";
import axios from "axios";

import { useAppContext } from "@context/AppContext";

import styles from "@styles/Header.module.scss";
import usersIcon from "@icons/users.png";

import endpoints from "@services/Api/endpoints";

import Navbar from "@components/Navbar";
import Users from "@containers/Users";

export default function Header() {
    const route = useRouter();
    const { state, setCurrentPage, setMessage } = useAppContext();
    const [showUsers, setShowUsers] = useState(false);
    const [showOptUsers, setShowOptUsers] = useState(true);
    const [currentUser, setCurrentUser] = useState(null);

    useEffect(() => {
        async function getCurrentUser() {
            const url = `${endpoints.users.get}?email=${state.user.email}`;
            await axios
                .get(url)
                .then((response) => {
                    const users = response.data.users;
                    if (users.length > 0) {
                        setCurrentUser(users[0]);
                        if (users[0].role.value != "administrador") {
                            setShowOptUsers(false);
                        }
                    }
                })
                .catch((error) => {
                    console.error(error);
                    setMessage({
                        type: "error",
                        message: "No se pudo obtener el usuario logueado",
                    });
                });
        }

        if (currentUser == null) {
            getCurrentUser();
        }
    }, [currentUser]);

    return (
        <header className={styles.Header}>
            <div>
                <div
                    style={{ cursor: "pointer" }}
                    onClick={() => {
                        setCurrentPage("");
                        route.push("/dashboard");
                    }}>
                    <Image
                        className={styles.Header__logo}
                        src={process.env.REACT_APP_LOGO2_URL}
                        alt='Logo2'
                        height={60}
                        width={60}
                    />
                </div>

                {state.currentPage != "" ? (
                    <h3 className={styles.Header__title}>
                        {"ATI - " + state.currentPage}
                    </h3>
                ) : (
                    <h3 className={styles.Header__title}>
                        {process.env.REACT_APP_HEADER}
                    </h3>
                )}
            </div>
            <div>
                {showOptUsers ? (
                    <Image
                        className='Header__Option'
                        src={usersIcon}
                        alt='Users'
                        height={25}
                        width={35}
                        onClick={() => setShowUsers(true)}
                        title='Usuarios'
                    />
                ) : null}

                <Navbar />
            </div>
            {showUsers ? (
                <Users closeModal={() => setShowUsers(false)} />
            ) : null}
        </header>
    );
}
