"use client";

import React from "react";
import { googleLogout } from "@react-oauth/google";
import Cookies from "js-cookie";
import Link from "next/link";
import Image from "next/image";
import { useRouter } from "next/navigation";

import { useAppContext } from "@context/AppContext";

import MenuOptions from "@utils/MenuOptions";
import logoutIcon from "@img/logout.png";

const Navbar = () => {
    const { state, setUser } = useAppContext();
    const router = useRouter();

    const collapseMenu = (e) => {
        const collapse = e.target;
        let parent = collapse.parentElement;
        parent.classList.toggle("active");

        let content = collapse.nextElementSibling;

        if (content.style.maxHeight) {
            content.style.maxHeight = null;
        } else {
            content.style.maxHeight = content.scrollHeight + "px";
        }
    };

    const logout = async () => {
        googleLogout();
        Cookies.set("loggedUser", null);
        router.push("/login");
    };

    const showMenu = () => {
        document.querySelector(".Profile__card").classList.toggle("Show");
    };

    return (
        <nav className='Navbar'>
            <div className='Navbar__profile'>
                <div className='Menu'>
                    <p className='Profile__username'>{state.user.name}</p>
                    <img
                        className='Menu__toggle'
                        src={state.user.picture}
                        alt={state.user.name}
                        onClick={() => showMenu()}
                    />
                </div>
                <div className='Profile__card'>
                    <div className='Card__title'>
                        <p>{state.user.email}</p>
                    </div>
                    <div className='Card__content'>
                        <img src={state.user.picture} />
                        <h4>¡Hola, {state.user.given_name}!</h4>
                        <div className='Card__footer'>
                            <div className='Card__menu'>
                                <div className='MenuItem'>
                                    <Link href='/dashboard'>Inicio</Link>
                                </div>
                                {MenuOptions.map((option) =>
                                    option.children ? (
                                        <div
                                            className='MenuItem'
                                            key={option.id}>
                                            <div
                                                className='MenuCollapsible'
                                                key={option.id}
                                                onClick={(e) =>
                                                    collapseMenu(e)
                                                }>
                                                {option.title}
                                            </div>
                                            <div className='MenuCollapsible__content'>
                                                {option.children.map(
                                                    (child) => (
                                                        <div
                                                            className='MenuItem__child'
                                                            key={child.url}>
                                                            <Link
                                                                href={
                                                                    child.url
                                                                }>
                                                                {child.title}
                                                            </Link>
                                                        </div>
                                                    )
                                                )}
                                            </div>
                                        </div>
                                    ) : (
                                        <div
                                            className='MenuItem'
                                            key={option.id}>
                                            <Link href={option.toRoute}>
                                                {option.title}
                                            </Link>
                                        </div>
                                    )
                                )}
                            </div>
                            <div className='Logout' onClick={() => logout()}>
                                <Image
                                    src={logoutIcon}
                                    alt='Cerrar sesion'
                                    width={30}
                                    height={30}
                                />
                                <span>Cerrar sesión</span>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        </nav>
    );
    // }
};

export default Navbar;
