import React, { Component } from "react";
import axios from "axios";

import AppContext from "@context/AppContext";

import endpoints from "@services/Api/endpoints";

import NoData from "@components/NoData";
import Table from "@components/Table";
import FrmRole from "@components/FrmRole";

export default class RolesTable extends Component {
    constructor() {
        super();

        this.state = {
            showFrmRoles: false,
            role: null,
            roles: [],
            rows: [],
            columns: [
                {
                    width: 100,
                    name: "ID",
                    filter: null,
                },
                {
                    width: 300,
                    name: "Nombre",
                    filter: null,
                },
                {
                    width: 300,
                    name: "Valor",
                    filter: null,
                },
            ],
        };
    }

    static contextType = AppContext;

    getRoleRow(role) {
        return {
            id: role.id,
            rowClass: "",
            cells: [
                {
                    id: Math.random() * 100,
                    width: 100,
                    value: role.id,
                    isEditor: true,
                    onClick: () => {
                        this.setState({ role: role, showFrmRoles: true });
                    },
                },
                {
                    id: Math.random() * 100,
                    width: 300,
                    value: role.title,
                },
                {
                    id: Math.random() * 100,
                    width: 300,
                    value: role.value,
                },
            ],
        };
    }

    addRoleRow(role) {
        this.setState({
            roles: [role, ...this.state.roles],
            rows: [this.getRoleRow(role), ...this.state.rows],
        });
    }

    removeRoleRow(role) {
        const newRoles = this.state.roles.filter(
            (oldRole) => oldRole.id != role.id
        );

        const newRows = this.state.rows.filter(
            (oldRow) => oldRow.id != role.id
        );

        this.setState({ roles: newRoles, rows: newRows });
    }

    updateRoleRow(role) {
        const updatedRolesRow = this.state.rows.map((row) => {
            if (row.id == role.id) {
                return this.getRoleRow(role);
            }

            return row;
        });

        this.setState({ rows: updatedRolesRow });
    }

    async fetchRoles() {
        await axios
            .get(endpoints.roles.get)
            .then((response) => {
                const roles = response.data.roles;

                const rolesRows = roles.map((role) => this.getRoleRow(role));

                this.setState({ roles: roles, rows: rolesRows });
            })
            .catch((error) => {
                console.error(error);
                this.context.setMessage({
                    type: "error",
                    message: error.response.data.message,
                });
            });
    }

    componentDidMount() {
        this.fetchRoles();
    }

    render() {
        return (
            <div className='RolesTable'>
                <section className='Page__Container'>
                    <div className='Page__Options'>
                        <div
                            className='Option'
                            onClick={() =>
                                this.setState({
                                    showFrmRoles: true,
                                    role: null,
                                })
                            }>
                            Nuevo
                        </div>
                    </div>
                    <div className='Page__Wrapper'>
                        {this.state.roles.length == 0 ? (
                            <NoData message={"Sin registro de roles"} />
                        ) : null}
                        <div className='Content'>
                            <div className='Tbl__Scrollable'>
                                <Table
                                    columns={this.state.columns}
                                    rows={this.state.rows}
                                />
                            </div>

                            {this.state.showFrmRoles ? (
                                <FrmRole
                                    role={this.state.role}
                                    addRoleRow={(role) => this.addRoleRow(role)}
                                    updateRoleRow={(role) =>
                                        this.updateRoleRow(role)
                                    }
                                    removeRoleRow={(role) =>
                                        this.removeRoleRow(role)
                                    }
                                    closeModal={() =>
                                        this.setState({ showFrmRoles: false })
                                    }
                                />
                            ) : null}
                        </div>
                    </div>
                </section>
            </div>
        );
    }
}
