import React, { Component } from "react";
import Image from "next/image";

import filterIcon from "@icons/filter.png";
import editIcon from "@icons/edit.png";

import FilterDialog from "@components/FilterDialog";

export default class Table extends Component {
    constructor() {
        super();

        this.state = {
            filterTitle: "",
            appliedFilters: [],
            filters: [],
            setFilters: null,
            showFilter: false,
        };
    }

    showFilterDialog(filter) {
        this.setState({
            filterTitle: filter.filterTitle,
            appliedFilters: filter.appliedFilters,
            filters: filter.filters,
            setFilters: filter.setFilters,
            showFilter: true,
        });
    }

    showFilter(show) {
        this.setState({
            ...this.state,
            showFilter: show,
        });
    }

    render() {
        return (
            <div className='Table'>
                <div className='THeader'>
                    {this.props.columns.map((column) => (
                        <div
                            className='Cell'
                            style={{
                                width: `${column.width}px`,
                            }}
                            key={column.name}>
                            {column.name}
                            {column.filter && (
                                <div
                                    className={`Filter ${
                                        column.filter.appliedFilters.length > 0
                                            ? "Filter__Applied"
                                            : ""
                                    } ${
                                        column.filter.filters.length == 0
                                            ? "Disabled"
                                            : ""
                                    }`}
                                    onClick={
                                        column.filter.filters.length > 0
                                            ? () =>
                                                  this.showFilterDialog(
                                                      column.filter
                                                  )
                                            : null
                                    }>
                                    <Image
                                        src={filterIcon}
                                        alt='Filter'
                                        width={30}
                                        height={30}
                                    />
                                </div>
                            )}
                        </div>
                    ))}
                </div>
                <div
                    className={`Body ${this.props.sticky ? "Sticky" : ""}`}
                    onScroll={this.props.onscroll}>
                    {this.props.rows.map((row) => (
                        <div className={`Row ${row.rowClass}`} key={row.id}>
                            {row.cells.map((cell) => (
                                <div
                                    className={`Cell ${
                                        cell.onClick && !cell.isEditor
                                            ? "Hover"
                                            : ""
                                    }`}
                                    style={{
                                        width: `${cell.width}px`,
                                    }}
                                    key={cell.id}
                                    onClick={cell.onClick}>
                                    {cell.link && cell.value ? (
                                        <a href={cell.value} target='_black'>
                                            {cell.value}
                                        </a>
                                    ) : (
                                        <div
                                            style={{
                                                display: "flex",
                                                flexDirection: "column",
                                            }}
                                            className={`${
                                                cell.onClick && cell.isEditor
                                                    ? "Flex"
                                                    : ""
                                            }`}>
                                            <span>{cell.value}</span>
                                            {cell.isEditor && (
                                                <Image
                                                    alt='Edit'
                                                    className='Hover'
                                                    src={editIcon}
                                                    onClick={cell.onClick}
                                                    width={30}
                                                    height={30}
                                                />
                                            )}
                                        </div>
                                    )}
                                </div>
                            ))}
                        </div>
                    ))}
                </div>

                {this.state.showFilter && (
                    <FilterDialog
                        filterTitle={this.state.filterTitle}
                        appliedFilters={this.state.appliedFilters}
                        filters={this.state.filters}
                        setFilters={this.state.setFilters}
                        closeDialog={() => this.showFilter(false)}
                    />
                )}
            </div>
        );
    }
}
