import React, { Component } from "react";
import axios from "axios";
import Chart from "chart.js/auto";
import DataLabelsPlugin from "chartjs-plugin-datalabels";
import MonthlyChart from "@components/MonthlyChart";
import YearlyChart from "@components/YearlyChart";

export default class ChartsContainer extends Component {
    constructor() {
        super();

        this.state = {
            data: [],
        };
    }

    componentDidMount() {
        Chart.register(DataLabelsPlugin);

        try {
            const url = this.props.url + "?page=1&limit=" + this.props.limit;
            axios.get(url).then((response) => {
                this.setState({
                    data: response.data.data,
                });
            });
        } catch (error) {
            console.error(error);
        }
    }
    render() {
        return (
            <div className='Charts Dialog'>
                <div className='Dialog__Content Dialog__Content'>
                    <div className='Dialog__Title'>
                        {this.props.title ?? "Estadíssticas de registros"}
                    </div>
                    <div className='Dialog__Body'>
                        {this.state.data.length > 0 && (
                            <MonthlyChart
                                data={this.state.data}
                                label={this.props.label ?? "Datos"}
                                switchStatus={this.props.switchStatus}
                            />
                        )}

                        {this.state.data.length > 0 && (
                            <YearlyChart
                                data={this.state.data}
                                years={this.props.years}
                                label={this.props.label ?? "Datos"}
                                switchStatus={this.props.switchStatus}
                            />
                        )}
                    </div>
                    <div className='Dialog__Footer'>
                        <div
                            className='Button Button__primary'
                            onClick={() => this.props.closeModal()}>
                            Cerrar
                        </div>
                    </div>
                </div>
            </div>
        );
    }
}
