import React, { Component } from "react";

import RolesTable from "@components/RolesTable";
import UsersTable from "@components/UsersTable";

export default class Users extends Component {
    constructor() {
        super();

        this.state = {
            currentTabContent: <div></div>,
            options: [
                {
                    id: 1,
                    label: "Usuarios",
                    selected: true,
                    content: <UsersTable />,
                },
                {
                    id: 2,
                    label: "Roles",
                    selected: false,
                    content: <RolesTable />,
                },
            ],
        };
    }

    selectOption(selectedOption) {
        let currentContent = <div></div>;
        const updatedOption = this.state.options.map((option) => {
            if (option.id == selectedOption.id) {
                option.selected = true;
                currentContent = option.content;
                return option;
            }

            option.selected = false;
            return option;
        });

        this.setState({
            options: updatedOption,
            currentTabContent: currentContent,
        });
    }

    loadContent() {
        const currentContent = this.state.options.filter(
            (option) => option.selected
        );

        if (currentContent.length > 0) {
            this.setState({ currentTabContent: currentContent[0].content });
        }
    }

    componentDidMount() {
        this.loadContent();
    }

    render() {
        return (
            <div className='Users Dialog'>
                <div className='Dialog__Content'>
                    <div className='Dialog__Title'>
                        <p>Usuarios</p>
                        <div
                            className='Button Button__secondary'
                            onClick={() => this.props.closeModal()}>
                            Cerrar
                        </div>
                    </div>
                    <div className='Dialog__Body'>
                        <div className='Dialog__Options'>
                            {this.state.options.map((option) => (
                                <div
                                    key={option.id}
                                    className={`Dialog__Option ${
                                        option.selected ? "Active" : ""
                                    }`}
                                    onClick={() => this.selectOption(option)}>
                                    {option.label}
                                </div>
                            ))}
                        </div>
                        {this.state.currentTabContent}
                    </div>
                </div>
            </div>
        );
    }
}
