"use client";

import { useEffect, useState } from "react";
import axios from "axios";

import FrmNotification from "@components/FrmNotification";
import FrmUpdateStatus from "@components/FrmUpdateStatus";
import Legend from "@components/Legend";
import NoData from "@components/NoData";
import Table from "@components/Table";

import ChartsContainer from "@containers/ChartsContainer";
import { useAppContext } from "@context/AppContext";
import endpoints from "@services/Api/endpoints";

import RowColor from "@utils/RowColor";
import Scroll from "@utils/Scroll";
import FrmRemark from "@components/FrmComment";
import FrmEducation from "@components/FrmEducation";
import DateFormatter from "@utils/DateFormatter";
import FrmAssignUser from "@components/FrmAssignUser";
import TelegramChannels from "@utils/TelegramChannels";

export default function Education() {
    const now = new Date();
    const [educations, setEducations] = useState([]);
    const { state, setCurrentPage } = useAppContext();

    const [showFrmNotification, setShowFrmNotification] = useState(false);
    const [showCharts, setShowCharts] = useState(false);
    const [showFrmAssignUser, setShowFrmAssignUser] = useState(false);
    const [showFrmEducation, setShowFrmEducation] = useState(false);
    const [showFrmUpdateStatus, setShowFrmUpdateStatus] = useState(false);
    const [showFrmRemark, setShowFrmRemark] = useState(false);

    const [registry, setRegistry] = useState(null);
    const [totalItems, setTotalItems] = useState(0);
    const [educationId, setEducationId] = useState(null);
    const [assignationType, setAssignationType] = useState("");
    const [userType, setUserType] = useState("");

    // Paginator handlers vars
    const [pages, setPages] = useState(0);
    const [itemsCount, setItemsCount] = useState(0);
    const [perPage, setPerPage] = useState(20);
    const [currPage, setCurrPage] = useState(1);
    const [prevPage, setPrevPage] = useState(0);
    const [nextPage, setNextPage] = useState(0);
    const [itemsOf, setItemsOf] = useState(1);
    const [itemsTo, setItemsTo] = useState(0);
    const [toPage, setToPage] = useState(1);

    // URL
    const [url, setUrl] = useState(
        `${endpoints.education.get}?page=${toPage}&limit=${perPage}`
    );

    const [urlFilter, setUrlFilter] = useState(
        `${endpoints.education.filters}`
    );
    const [urlAssignUser, setUrlAssignUser] = useState("");

    // Filter options
    const filtroMes = [
        { mes: "Todos", no: 0 },
        { mes: "Enero", no: 1 },
        { mes: "Febrero", no: 2 },
        { mes: "Marzo", no: 3 },
        { mes: "Abril", no: 4 },
        { mes: "Mayo", no: 5 },
        { mes: "Junio", no: 6 },
        { mes: "Julio", no: 7 },
        { mes: "Agosto", no: 8 },
        { mes: "Septiembre", no: 9 },
        { mes: "Octubre", no: 10 },
        { mes: "Noviembre", no: 11 },
        { mes: "Diciembre", no: 12 },
    ];
    const [filtroAnios, setFiltroAnios] = useState(["Todos"]);

    const updateData = (response) => {
        setEducations(response.data.data);
        setPages(response.data.pages);
        setItemsCount(response.data.itemsCount);
        setTotalItems(
            response.data.itemsCount > totalItems
                ? response.data.itemsCount
                : totalItems
        );
        setCurrPage(response.data.currentPage);
        setPerPage(response.data.perPage);
        setPrevPage(response.data.prevPage);
        setNextPage(response.data.nextPage);

        setItemsOf(
            response.data.currentPage > 1
                ? perPage * (response.data.currentPage - 1) + 1
                : 1
        );
        let newItemsTo = response.data.perPage * response.data.currentPage;
        setItemsTo(newItemsTo);

        if (newItemsTo > response.data.itemsCount) {
            setItemsTo(response.data.itemsCount);
        }
    };

    const getFilters = () => {
        let queryParams = "";

        const txtMonth = document.getElementById("month");
        const txtYear = document.getElementById("year");

        const month = parseInt(txtMonth.value);
        const year = txtYear.value;

        queryParams +=
            month > 0 ? `&month=${month < 10 ? "0" + month : month}` : "";
        queryParams +=
            year != "Todos"
                ? `&year=${year}`
                : month > 0
                ? `&year=${now.getFullYear()}`
                : "";

        return queryParams;
    };

    const fetchData = () => {
        useEffect(() => {
            async function getData() {
                const response = await axios.get(url);
                updateData(response);
            }

            getData().then(() => {
                setCurrentPage("Educación");
                document.title = process.env.REACT_APP_TITLE + " - Educación";
            });
        }, [url]);
    };

    const fetchFilters = () => {
        useEffect(() => {
            async function getFilters() {
                const response = await axios.get(urlFilter);

                setFiltroAnios(response.data.filtroAnios);
            }

            getFilters();
        }, [urlFilter]);
    };

    const updateUrl = () => {
        let txtPerPage = document.getElementById("perPage");
        let txtToPage = document.getElementById("currentPage");
        if (txtPerPage && txtToPage) {
            let newUrl = `${endpoints.education.get}?page=${parseInt(
                txtToPage.value
            )}&limit=${parseInt(txtPerPage.value)}`;
            let newUrlFilter = `${endpoints.education.filters}?page=${parseInt(
                txtToPage.value
            )}&limit=${parseInt(txtPerPage.value)}`;

            const filters = getFilters();
            newUrl += filters;
            newUrlFilter += filters;

            setUrl(newUrl);
            setUrlFilter(newUrlFilter);
        }
    };

    const onChangePage = (e) => {
        const value = e.target.value;

        if (value !== "" && regexpNumber.test(value)) {
            const toPage = parseInt(value);
            if (toPage > pages) {
                e.target.value = pages;
            }

            if (toPage > 0) {
                updateUrl();
            }
        }
    };

    const onChangeItemsPerPage = (e) => {
        const value = e.target.value;

        if (value !== "" && regexpNumber.test(value)) {
            const itemsPerPage = parseInt(value);

            if (itemsPerPage > 0) {
                setPerPage(value);
                setToPage(1);
                updateUrl();
            }
        }
    };

    const onChangeSelection = (e) => {
        document.getElementById("currentPage").value = 1;
        updateUrl();
    };

    const getPage = (toPage) => {
        document.getElementById("currentPage").value = toPage;
        updateUrl();
    };

    fetchFilters();
    fetchData();

    const stickyColumns = [
        {
            width: 100,
            name: "ID",
            filter: null,
        },
        {
            width: 200,
            name: "Nombre del cliente",
            filter: null,
        },
    ];
    const stickyRows = educations.map((education) => {
        return {
            id: education.id,
            rowClass: RowColor.getColor(education.estatus),
            cells: [
                {
                    id: Math.random() * 100,
                    width: 100,
                    value: education.id,
                },
                {
                    id: Math.random() * 100,
                    width: 200,
                    value: education.nombre_cliente,
                },
            ],
        };
    });

    const scrollableColumns = [
        {
            width: 150,
            name: "Marca temporal",
            filter: null,
        },
        {
            width: 250,
            name: "Vendedor",
            filter: null,
        },
        {
            width: 250,
            name: "Preventa asignado",
            filter: null,
        },
        {
            width: 250,
            name: "Acompañante",
            filter: null,
        },
        {
            width: 200,
            name: "Nombre vendedor",
            filter: null,
        },
        {
            width: 200,
            name: "Insitución Educativa",
            filter: null,
        },
        {
            width: 350,
            name: "Descripción general de la institución educativa (nivel educativo, tamaño, Aprox)",
            filter: null,
        },
        {
            width: 450,
            name: "Cuál es el principal desafío o problema que la institución educativa enfrenta y que desea resolver con nuestra solución tecnológica?",
            filter: null,
        },
        {
            width: 450,
            name: "¿Cuáles son los objetivos educativos y estratégicos de la institución relacionados con esta solución?",
            filter: null,
        },
        {
            width: 250,
            name: "¿Cuál es el presupuesto estimado para esta solución?",
            filter: null,
        },
        {
            width: 400,
            name: "¿Cuenta con la infraestructura adecuada para su solucion? (redes, servidores, dispositivos, software utilizado, etc.)",
            filter: null,
        },
        {
            width: 300,
            name: "¿Que tecnologias tienen destinadas a las aulas actualmente?",
            filter: null,
        },
        {
            width: 450,
            name: "Necesidad principal?",
            filter: null,
        },
        {
            width: 450,
            name: "¿Cuáles son los criterios de evaluación y selección que la institución educativa utilizará para tomar una decisión final?",
            filter: null,
        },
        {
            width: 350,
            name: "¿La institución educativa ha considerado otras soluciones o proveedores? Si es así, ¿cuáles?",
            filter: null,
        },
        {
            width: 500,
            name: "¿Hay alguna otra información adicional que consideres relevante para el desarrollo de la solución tecnológica en el entorno educativo?",
            filter: null,
        },
        {
            width: 350,
            name: "Link carpeta",
            filter: null,
        },
        {
            width: 100,
            name: "Estatus",
            filter: null,
        },
        {
            width: 150,
            name: "Observaciones",
            filter: null,
        },
    ];
    const scrollableRows = educations.map((education) => {
        return {
            id: education.id,
            rowClass: RowColor.getColor(education.estatus),
            cells: [
                {
                    id: Math.random() * 100,
                    width: 150,
                    value: DateFormatter.fullDateNormal(
                        education.marca_temporal
                    ),
                },
                {
                    id: Math.random() * 100,
                    width: 250,
                    value: education.usuario,
                },
                {
                    id: Math.random() * 100,
                    width: 250,
                    value: education.vendedor ?? "-",
                    onClick: education.vendedor
                        ? null
                        : () => {
                              setAssignationType("Vendedor");
                              setUserType("seller");
                              setRegistry(education);
                              setUrlAssignUser(
                                  endpoints.education.assign_seller
                              );
                              setShowFrmAssignUser(true);
                          },
                },
                {
                    id: Math.random() * 100,
                    width: 250,
                    value: education.seguidor ?? "-",
                    onClick: () => {
                        setAssignationType("Acompañante");
                        setUserType("follower");
                        setRegistry(education);
                        setUrlAssignUser(endpoints.education.assign_follower);
                        setShowFrmAssignUser(true);
                    },
                },
                {
                    id: Math.random() * 100,
                    width: 200,
                    value: education.nombre_vendedor,
                },
                {
                    id: Math.random() * 100,
                    width: 200,
                    value: education.nombre_institucion,
                },
                {
                    id: Math.random() * 100,
                    width: 350,
                    value: education.descripcion,
                },
                {
                    id: Math.random() * 100,
                    width: 450,
                    value: education.desafio,
                },
                {
                    id: Math.random() * 100,
                    width: 450,
                    value: education.objetivos,
                },
                {
                    id: Math.random() * 100,
                    width: 250,
                    value: education.presupuesto,
                },
                {
                    id: Math.random() * 100,
                    width: 400,
                    value: education.infraestructura_ti,
                },
                {
                    id: Math.random() * 100,
                    width: 300,
                    value: education.integraciones,
                },
                {
                    id: Math.random() * 100,
                    width: 450,
                    value: education.necesidades,
                },
                {
                    id: Math.random() * 100,
                    width: 450,
                    value: education.criterios,
                },
                {
                    id: Math.random() * 100,
                    width: 350,
                    value: education.otras_soluciones,
                },
                {
                    id: Math.random() * 100,
                    width: 500,
                    value: education.informacion_adicional,
                },
                {
                    id: Math.random() * 100,
                    width: 350,
                    value: education.link_carpeta ?? "",
                },
                {
                    id: Math.random() * 100,
                    width: 100,
                    value: education.estatus,
                    onClick:
                        education.estatus != null &&
                        education.estatus != "" &&
                        education.estatus != "Finalizado"
                            ? () => {
                                  setRegistry(education);
                                  setShowFrmUpdateStatus(true);
                              }
                            : null,
                },
                {
                    id: Math.random() * 100,
                    width: 150,
                    value: education.observaciones,
                    onClick: () => {
                        setRegistry(education);
                        setShowFrmRemark(true);
                    },
                },
            ],
        };
    });

    return (
        <section className='Page__Container'>
            <div className='Page__Options'>
                <div
                    className='Option'
                    onClick={() => setShowFrmNotification(true)}>
                    Notificaciones
                </div>
                <div className='Option' onClick={() => setShowCharts(true)}>
                    Gráficas
                </div>
                <div
                    className='Option'
                    onClick={() => setShowFrmEducation(true)}>
                    Formulario
                </div>

                {/* OPTIONS MODALS */}

                {showFrmNotification ? (
                    <FrmNotification
                        title={"Registrar correo"}
                        url={endpoints.email_notification.store}
                        closeModal={() => setShowFrmNotification(false)}
                    />
                ) : null}

                {showCharts ? (
                    <ChartsContainer
                        title={"Estadísticas de educaciones"}
                        url={endpoints.education.get}
                        limit={totalItems}
                        years={filtroAnios}
                        label={"Educaciones"}
                        switchStatus={true}
                        closeModal={() => setShowCharts(false)}
                    />
                ) : null}

                {showFrmEducation ? (
                    <FrmEducation
                        updateData={updateData}
                        url={url}
                        closeModal={() => setShowFrmEducation(false)}
                    />
                ) : null}
            </div>
            <div className='Page__Wrapper'>
                {educations.length === 0 ? (
                    <NoData message={"Sin registro de educaciones"} />
                ) : null}
                <div className='Content'>
                    <div className='Sticky__Container'>
                        <Table
                            onscroll={() =>
                                Scroll.onscroll(
                                    ".Page__Wrapper .Content .Sticky__Container .Table .Body",
                                    ".Page__Wrapper .Content .Tbl__Scrollable .Table .Body"
                                )
                            }
                            columns={stickyColumns}
                            rows={stickyRows}
                        />
                    </div>
                    <div className='Tbl__Scrollable'>
                        <Table
                            onscroll={() =>
                                Scroll.onscroll(
                                    ".Page__Wrapper .Content .Tbl__Scrollable .Table .Body",
                                    ".Page__Wrapper .Content .Sticky__Container .Table .Body"
                                )
                            }
                            columns={scrollableColumns}
                            rows={scrollableRows}
                        />
                    </div>

                    {/* REGISTRY MODALS */}

                    {showFrmUpdateStatus ? (
                        <FrmUpdateStatus
                            registry={registry}
                            url={endpoints.education.estatus.update}
                            closeModal={() => setShowFrmUpdateStatus(false)}
                            urlData={url}
                            updateData={updateData}
                            type={"statusp"}
                            entity={"education"}
                        />
                    ) : null}

                    {showFrmRemark ? (
                        <FrmRemark
                            registry={registry}
                            closeModal={() => setShowFrmRemark(false)}
                            url={endpoints.education.estatus.remarks}
                            urlData={url}
                            updateData={updateData}
                        />
                    ) : null}

                    {showFrmAssignUser ? (
                        <FrmAssignUser
                            type={assignationType}
                            url={urlAssignUser}
                            registry={registry}
                            closeModal={() => setShowFrmAssignUser(false)}
                            urlData={url}
                            updateData={updateData}
                            entityType={"education"}
                            userType={userType}
                            telegramChannel={TelegramChannels.educationChannel}
                        />
                    ) : null}
                </div>

                <div className='Paginator'>
                    <Legend />
                    {/* FILTERS BY DATE */}
                    <div className='DateFilters'>
                        <div className='Filter'>
                            <label htmlFor='month'>Filtrar por mes</label>
                            <select
                                name='month'
                                id='month'
                                onChange={() => onChangeSelection()}>
                                {filtroMes.map((option) => (
                                    <option value={option.no} key={option.no}>
                                        {option.mes}
                                    </option>
                                ))}
                            </select>
                        </div>

                        <div className='Filter'>
                            <label htmlFor='year'>Año</label>
                            <select
                                name='year'
                                id='year'
                                onChange={() => onChangeSelection()}>
                                {filtroAnios.map((anio) => (
                                    <option value={anio} key={anio}>
                                        {anio}
                                    </option>
                                ))}
                            </select>
                        </div>
                    </div>

                    {/* PAGINATOR CONTROLS */}
                    <div className='Paginator__Controls'>
                        <div className='Paginator__CurrentPage'>
                            <label htmlFor='currentPage'>Página</label>
                            <input
                                type='text'
                                name='currentPage'
                                id='currentPage'
                                defaultValue={currPage}
                                onChange={(e) => onChangePage(e)}
                            />
                        </div>

                        <div className='Paginator__RecordsShown'>
                            <label htmlFor='perPage'>
                                Registros por página
                            </label>
                            <input
                                type='text'
                                name='perPage'
                                id='perPage'
                                defaultValue={perPage}
                                onChange={(e) => onChangeItemsPerPage(e)}
                            />
                        </div>

                        <div className='Paginator__SomeOfLabels'>
                            {itemsOf}-{itemsTo} de {itemsCount}
                        </div>

                        <div className='Paginator__Buttons'>
                            <span
                                className={`Prev__Button ${
                                    prevPage ? "" : "disabled"
                                }`}
                                onClick={
                                    prevPage ? () => getPage(prevPage) : null
                                }>
                                {"<"}
                            </span>
                            <span
                                className={`Next__Button ${
                                    nextPage ? "" : "disabled"
                                }`}
                                onClick={
                                    nextPage ? () => getPage(nextPage) : null
                                }>
                                {">"}
                            </span>
                        </div>
                    </div>
                </div>
            </div>
        </section>
    );
}
