"use client";

import React, { useEffect, useState } from "react";
import axios from "axios";
import { useAppContext } from "@context/AppContext";
import endpoints from "@services/Api/endpoints";

import Head from "next/head";

import Table from "@components/Table";
import NoData from "@components/NoData";
import FrmNotification from "@components/FrmNotification";
import LstHistory from "@components/LstHistory";
import FrmBid from "@components/FrmBid";
import RegistryHistoryDetail from "@components/RegistryHistoryDetail";
import FrmUpdateStatus from "@components/FrmUpdateStatus";
import RowColor from "@utils/RowColor";
import DateFormatter from "@utils/DateFormatter";
import FrmUpdateDate from "@components/FrmUpdateDate";
import ChartsContainer from "@containers/ChartsContainer";
import FrmRemark from "@components/FrmComment";
import FrmAssignUser from "@components/FrmAssignUser";
import Scroll from "@utils/Scroll";
import BidLegend from "@components/BidLegend";
import TelegramChannels from "@utils/TelegramChannels";
import FrmProductSatus from "@components/FrmProductSatus";

const Bid = () => {
    const now = new Date();
    const regexpNumber = /^[0-9\b]+$/;
    const { state, setCurrentPage } = useAppContext();
    const [bids, setBids] = useState([]);
    const [showFrmNotification, setShowFrmNotification] = useState(false);
    const [showLstHistory, setShowLstHistory] = useState(false);
    const [showFrmBid, setShowFrmBid] = useState(false);
    const [showRegistryHistory, setShowRegistryHistory] = useState(false);
    const [showFrmUpdateStatus, setShowFrmUpdateStatus] = useState(false);
    const [showFrmUpdateDate, setShowFrmUpdateDate] = useState(false);
    const [showFrmRemark, setShowFrmRemark] = useState(false);
    const [showCharts, setShowCharts] = useState(false);
    const [showFrmAssignUser, setShowFrmAssignUser] = useState(false);
    const [showFrmProductStatus, setShowFrmProductStatus] = useState(false);

    const [registry, setRegistry] = useState(null);
    const [urlUpdate, setUrlUpdate] = useState("");
    const [urlAssignUser, setUrlAssignUser] = useState("");
    const [statusType, setStatusType] = useState("");
    const [totalItems, setTotalItems] = useState(0);
    const [assignationType, setAssignationType] = useState("Vendedor");
    const [userType, setUserType] = useState("");

    // Paginator handlers vars
    const [pages, setPages] = useState(0);
    const [itemsCount, setItemsCount] = useState(0);
    const [perPage, setPerPage] = useState(20);
    const [currPage, setCurrPage] = useState(1);
    const [prevPage, setPrevPage] = useState(0);
    const [nextPage, setNextPage] = useState(0);
    const [itemsOf, setItemsOf] = useState(1);
    const [itemsTo, setItemsTo] = useState(0);
    const [toPage, setToPage] = useState(1);

    // Available options
    const estatusFinal = [
        "Se gano",
        "Se perdio",
        "Deserto",
        "Se está trabajando",
        "En espera de fallo",
    ];

    // URL
    const [url, setUrl] = useState(
        `${endpoints.bid.get}?page=${toPage}&limit=${perPage}`
    );

    const [urlFilter, setUrlFilter] = useState(`${endpoints.bid.filters}`);

    // Filters options
    const [filtroPartidas, setFiltroPartidas] = useState([]);
    const [filtroMarcasTemporales, setFiltroMarcasTemporales] = useState([]);
    const [filtroNombresLicitaciones, setFiltroNombresLicitaciones] = useState(
        []
    );
    const [filtroDependenciasLicitantes, setFiltroDependenciasLicitantes] =
        useState([]);
    const [filtroDependenciaUsuarios, setFiltroDependenciaUsuarios] = useState(
        []
    );
    const [filtroEmpresasParticipantes, setFiltroEmpresasParticipantes] =
        useState([]);
    const [filtroFechasPublicacion, setFiltroFechasPublicacion] = useState([]);
    const [filtroLugaresPublicacion, setFiltroLugaresPublicacion] = useState(
        []
    );
    const [filtroFechasPreguntas, setFiltroFechasPreguntas] = useState([]);
    const [filtroHorasLimiteEnviop, setFiltroHorasLimiteEnviop] = useState([]);
    const [filtroTiposDeEntrega, setFiltroTiposDeEntrega] = useState([]);
    const [filtroDomiciliosCorreos, setFiltroDomiciliosCorreos] = useState([]);
    const [filtroFechasJuntaAcl, setFiltroFechasJuntaAcl] = useState([]);
    const [filtroHorasRegistroJA, setFiltroHorasRegistroJA] = useState([]);
    const [filtroHorasJA, setFiltroHorasJA] = useState([]);
    const [filtroJuntasObligatorias, setFiltroJuntasObligatorias] = useState(
        []
    );
    const [filtroDomiciliosJunta, setFiltroDomiciliosJunta] = useState([]);
    const [filtroTiposJuntas, setFiltroTiposJuntas] = useState([]);
    const [filtroDomicilios, setFiltroDomicilios] = useState([]);
    const [filtroFechasPre, setFiltroFechasPre] = useState([]);
    const [filtroHorasRegistroPre, setFiltroHorasRegistroPre] = useState([]);
    const [filtroHorasApertura, setFiltroHorasApertura] = useState([]);
    const [filtroRequierePreventa, setFiltroRequierePreventa] = useState([]);
    const [filtroPresentacion, setFiltroPresentacion] = useState([]);
    const [filtroPreguntas, setFiltroPreguntas] = useState([]);
    const [filtroEstatusFicha, setFiltroEstatusFicha] = useState([]);
    const filtroMes = [
        { mes: "Todos", no: 0 },
        { mes: "Enero", no: 1 },
        { mes: "Febrero", no: 2 },
        { mes: "Marzo", no: 3 },
        { mes: "Abril", no: 4 },
        { mes: "Mayo", no: 5 },
        { mes: "Junio", no: 6 },
        { mes: "Julio", no: 7 },
        { mes: "Agosto", no: 8 },
        { mes: "Septiembre", no: 9 },
        { mes: "Octubre", no: 10 },
        { mes: "Noviembre", no: 11 },
        { mes: "Diciembre", no: 12 },
    ];
    const [filtroAnios, setFiltroAnios] = useState(["Todos"]);

    // Filters applied
    const [filterByNombres, setFilterByNombres] = useState([]);
    const [filterByMarcaTemporal, setFilterByMarcaTemporal] = useState([]);
    const [filterByPartidas, setFilterByPartidas] = useState([]);
    const [filterByDependenciasLicitantes, setFilterByDependenciasLicitantes] =
        useState([]);
    const [filterByDependenciasUsuarios, setFilterByDependenciasUsuarios] =
        useState([]);
    const [filterByEmpresasParticipantes, setFilterByEmpresasParticipantes] =
        useState([]);
    const [filterByFechasPublicacion, setFilterByFechasPublicacion] = useState(
        []
    );
    const [filterByLugaresPublicacion, setFilterByLugaresPublicacion] =
        useState([]);
    const [filterByFechasEnvioPreguntas, setFilterByFechasEnvioPreguntas] =
        useState([]);
    const [filterByHorasLimiteEnviop, setFilterByHorasLimiteEnviop] = useState(
        []
    );
    const [filterByTiposDeEntrega, setFilterByTiposDeEntrega] = useState([]);
    const [filterByDomiciliosCorreos, setFilterByDomiciliosCorreos] = useState(
        []
    );
    const [filterByFechasJuntaAcl, setFilterByFechasJuntaAcl] = useState([]);
    const [filterByHorasRegistroJA, setFilterByHorasRegistroJA] = useState([]);
    const [filterByHorasJA, setFilterByHorasJA] = useState([]);
    const [filterByJuntasObligatorias, setFilterByJuntasObligatorias] =
        useState([]);
    const [filterByDomiciliosJunta, setFilterByDomiciliosJunta] = useState([]);
    const [filterByTiposJuntas, setFilterByTiposJuntas] = useState([]);
    const [filterByDomicilios, setFilterByDomicilios] = useState([]);
    const [filterByFechasPre, setFilterByFechasPre] = useState([]);
    const [filterByHorasRegistroPre, setFilterByHorasRegistroPre] = useState(
        []
    );
    const [filterByHorasApertura, setFilterByHorasApertura] = useState([]);
    const [filterByRequierePreventa, setFilterByRequierePreventa] = useState(
        []
    );
    const [filterByPresentacion, setFilterByPresentacion] = useState([]);
    const [filterByPreguntas, setFilterByPreguntas] = useState([]);
    const [filterByEstatusFicha, setFilterByEstatusFicha] = useState([]);
    const [filterByEstatusFinal, setFilterByEstatusFinal] = useState([]);

    const getQuery = (filters, name, isDate = false) => {
        let query = "";

        for (const filter of filters) {
            if (isDate) {
                if (name == "tmpMarca") {
                    const dateTime = filter.split(" ");
                    const date = dateTime[1].split("-");
                    const time = dateTime[0];

                    query += `&${name}[]=${date[2]}-${date[1]}-${date[0]} ${time}`;
                } else {
                    const splitDate = filter.split("-");
                    query += `&${name}[]=${splitDate[2]}-${splitDate[1]}-${splitDate[0]}`;
                }
            } else {
                query += `&${name}[]=${filter}`;
            }
        }

        return query;
    };

    const getFilters = () => {
        let queryParams = "";

        queryParams += getQuery(filterByPartidas, "partidas", false);
        queryParams += getQuery(filterByMarcaTemporal, "tmpMarca", true);
        queryParams += getQuery(filterByNombres, "nombre", false);
        queryParams += getQuery(
            filterByDependenciasLicitantes,
            "dependencia_lic",
            false
        );
        queryParams += getQuery(
            filterByDependenciasUsuarios,
            "dependencia_usu",
            false
        );
        queryParams += getQuery(
            filterByEmpresasParticipantes,
            "empresas",
            false
        );
        queryParams += getQuery(filterByFechasPublicacion, "fechas_pub", true);
        queryParams += getQuery(
            filterByLugaresPublicacion,
            "lugares_pub",
            false
        );
        queryParams += getQuery(
            filterByFechasEnvioPreguntas,
            "fecha_envio_pre",
            true
        );
        queryParams += getQuery(
            filterByHorasLimiteEnviop,
            "hora_limite_enviop",
            false
        );
        queryParams += getQuery(
            filterByTiposDeEntrega,
            "como_se_entrega",
            false
        );
        queryParams += getQuery(
            filterByDomiciliosCorreos,
            "domicilio_correo",
            false
        );
        queryParams += getQuery(
            filterByFechasJuntaAcl,
            "fecha_junta_acl",
            true
        );
        queryParams += getQuery(
            filterByHorasRegistroJA,
            "hora_registro_ja",
            false
        );
        queryParams += getQuery(filterByHorasJA, "hora_ja", false);
        queryParams += getQuery(
            filterByJuntasObligatorias,
            "junta_obligatoria",
            false
        );
        queryParams += getQuery(
            filterByDomiciliosJunta,
            "domicilio_junta",
            false
        );
        queryParams += getQuery(filterByTiposJuntas, "tipo_junta", false);
        queryParams += getQuery(filterByDomicilios, "domicilio", false);
        queryParams += getQuery(filterByFechasPre, "fecha_pre", true);
        queryParams += getQuery(
            filterByHorasRegistroPre,
            "hora_registro_pre",
            false
        );
        queryParams += getQuery(filterByHorasApertura, "hora_apertura", false);
        queryParams += getQuery(
            filterByRequierePreventa,
            "requiere_preventa",
            false
        );
        queryParams += getQuery(filterByPresentacion, "presentation_status");
        queryParams += getQuery(filterByPreguntas, "estatus_preguntas");
        queryParams += getQuery(filterByEstatusFicha, "estatus_ficha");
        queryParams += getQuery(filterByEstatusFinal, "final_status");

        const txtMonth = document.getElementById("month");
        const txtYear = document.getElementById("year");

        const month = parseInt(txtMonth.value);
        const year = txtYear.value;

        queryParams +=
            month > 0 ? `&month=${month < 10 ? "0" + month : month}` : "";
        queryParams +=
            year != "Todos"
                ? `&year=${year}`
                : month > 0
                ? `&year=${now.getFullYear()}`
                : "";

        return queryParams;
    };

    const updateData = (response) => {
        setBids(response.data.data);
        setPages(response.data.pages);
        setItemsCount(response.data.itemsCount);
        setTotalItems(
            response.data.itemsCount > totalItems
                ? response.data.itemsCount
                : totalItems
        );
        setCurrPage(response.data.currentPage);
        setPerPage(response.data.perPage);
        setPrevPage(response.data.prevPage);
        setNextPage(response.data.nextPage);

        setItemsOf(
            response.data.currentPage > 1
                ? perPage * (response.data.currentPage - 1) + 1
                : 1
        );
        let newItemsTo = response.data.perPage * response.data.currentPage;
        setItemsTo(newItemsTo);

        if (newItemsTo > response.data.itemsCount) {
            setItemsTo(response.data.itemsCount);
        }
    };

    const fetchData = () => {
        useEffect(() => {
            async function getData() {
                const response = await axios.get(url);
                updateData(response);
            }

            getData().then(() => {
                setCurrentPage("Licitaciones");
                document.title =
                    process.env.REACT_APP_TITLE + " - Licitaciones";
            });
        }, [url]);
    };

    const fetchFilters = () => {
        useEffect(() => {
            async function getFilters() {
                const response = await axios.get(urlFilter);

                setFiltroPartidas(response.data.filtroPartidas);
                setFiltroMarcasTemporales(response.data.filtroMarcasTemporales);
                setFiltroNombresLicitaciones(
                    response.data.filtroNombresLicitaciones
                );
                setFiltroDependenciasLicitantes(
                    response.data.filtroDependenciasLicitantes
                );
                setFiltroDependenciaUsuarios(
                    response.data.filtroDependenciaUsuarios
                );
                setFiltroEmpresasParticipantes(
                    response.data.filtroEmpresasParticipantes
                );
                setFiltroFechasPublicacion(
                    response.data.filtroFechasPublicacion
                );
                setFiltroLugaresPublicacion(
                    response.data.filtroLugaresPublicacion
                );
                setFiltroFechasPreguntas(response.data.filtroFechasPreguntas);
                setFiltroHorasLimiteEnviop(
                    response.data.filtroHorasLimiteEnviop
                );
                setFiltroTiposDeEntrega(response.data.filtroTiposDeEntrega);
                setFiltroDomiciliosCorreos(
                    response.data.filtroDomiciliosCorreos
                );
                setFiltroFechasJuntaAcl(response.data.filtroFechasJuntaAcl);
                setFiltroHorasRegistroJA(response.data.filtroHorasRegistroJA);
                setFiltroHorasJA(response.data.filtroHorasJA);
                setFiltroJuntasObligatorias(
                    response.data.filtroJuntasObligatorias
                );
                setFiltroDomiciliosJunta(response.data.filtroDomiciliosJunta);
                setFiltroTiposJuntas(response.data.filtroTiposJuntas);
                setFiltroDomicilios(response.data.filtroDomicilios);
                setFiltroFechasPre(response.data.filtroFechasPre);
                setFiltroHorasRegistroPre(response.data.filtroHorasRegistroPre);
                setFiltroHorasApertura(response.data.filtroHorasApertura);
                setFiltroRequierePreventa(response.data.filtroRequierePreventa);
                setFiltroPresentacion(response.data.filtroPresentacion);
                setFiltroPreguntas(response.data.filtroPreguntas);
                setFiltroEstatusFicha(response.data.filtroEstatusFicha);
                setFiltroAnios(response.data.filtroAnios);
            }

            getFilters();
        }, [urlFilter]);
    };

    // Apply filters on change
    useEffect(() => {
        updateUrl();
    }, [
        filterByNombres,
        filterByMarcaTemporal,
        filterByPartidas,
        filterByDependenciasLicitantes,
        filterByDependenciasUsuarios,
        filterByEmpresasParticipantes,
        filterByFechasPublicacion,
        filterByLugaresPublicacion,
        filterByFechasEnvioPreguntas,
        filterByHorasLimiteEnviop,
        filterByTiposDeEntrega,
        filterByDomiciliosCorreos,
        filterByFechasJuntaAcl,
        filterByHorasRegistroJA,
        filterByHorasJA,
        filterByJuntasObligatorias,
        filterByDomiciliosJunta,
        filterByTiposJuntas,
        filterByDomicilios,
        filterByFechasPre,
        filterByHorasRegistroPre,
        filterByHorasApertura,
        filterByRequierePreventa,
        filterByPresentacion,
        filterByPreguntas,
        filterByEstatusFicha,
        filterByEstatusFinal,
    ]);

    const updateUrl = () => {
        let txtPerPage = document.getElementById("perPage");
        let txtToPage = document.getElementById("currentPage");
        if (txtPerPage && txtToPage) {
            let newUrl = `${endpoints.bid.get}?page=${parseInt(
                txtToPage.value
            )}&limit=${parseInt(txtPerPage.value)}`;
            let newUrlFilter = `${endpoints.bid.filters}?page=${parseInt(
                txtToPage.value
            )}&limit=${parseInt(txtPerPage.value)}`;
            const filters = getFilters();
            newUrl += filters;
            newUrlFilter += filters;

            setUrl(newUrl);
            setUrlFilter(newUrlFilter);
        }
    };

    const onChangePage = (e) => {
        const value = e.target.value;

        if (value !== "" && regexpNumber.test(value)) {
            const toPage = parseInt(value);
            if (toPage > pages) {
                e.target.value = pages;
            }

            if (toPage > 0) {
                updateUrl();
            }
        }
    };

    const onChangeItemsPerPage = (e) => {
        const value = e.target.value;

        if (value !== "" && regexpNumber.test(value)) {
            const itemsPerPage = parseInt(value);

            if (itemsPerPage > 0) {
                setPerPage(value);
                setToPage(1);
                updateUrl();
            }
        }
    };

    const onChangeSelection = (e) => {
        document.getElementById("currentPage").value = 1;
        updateUrl();
    };

    const getPage = (toPage) => {
        document.getElementById("currentPage").value = toPage;
        updateUrl();
    };

    fetchFilters();
    fetchData();

    const updateFinalEstatus = (status) => {
        try {
            axios
                .post(endpoints.status.final, {
                    id: registry.id,
                    estatus: status,
                    usuarioFinal: state.user.name,
                })
                .then((response) => console.log("final", response));
        } catch (error) {
            console.error(error);
        }
    };

    const stikyColumns = [
        {
            width: 100,
            name: "ID",
            filter: null,
        },
        {
            width: 200,
            name: "Nombre de la licitación",
            filter: {
                appliedFilters: filterByNombres,
                filterTitle: "Filtrar por nombre de licitación",
                filters: filtroNombresLicitaciones,
                setFilters: setFilterByNombres,
            },
        },
    ];

    const stickyRows = bids.map((bid) => {
        return {
            id: bid.id,
            rowClass: RowColor.get(bid.estatus_final),
            cells: [
                {
                    id: Math.random() * 102,
                    width: 100,
                    value: bid.id,
                    onClick: () => {
                        setRegistry(bid);
                        setShowFrmBid(true);
                    },
                    isEditor: true,
                },
                {
                    id: Math.random() * 102,
                    width: 200,
                    value: bid.nombre,
                    onClick: () => {
                        setRegistry(bid);
                        setShowRegistryHistory(true);
                    },
                },
            ],
        };
    });

    const scrollableColumns = [
        {
            width: 150,
            name: "Marca temporal",
            filter: {
                appliedFilters: filterByMarcaTemporal,
                filterTitle: "Filtrar por marca temporal",
                filters: filtroMarcasTemporales,
                setFilters: setFilterByMarcaTemporal,
            },
        },
        {
            width: 250,
            name: "Vendedor",
            filter: null,
        },
        {
            width: 250,
            name: "Preventa asignado",
            filter: null,
        },
        {
            width: 250,
            name: "Acompañante",
            filter: null,
        },
        {
            width: 200,
            name: "Dependencia que licita",
            filter: {
                appliedFilters: filterByDependenciasLicitantes,
                filterTitle: "Filtrar por dependencias licitantes",
                filters: filtroDependenciasLicitantes,
                setFilters: setFilterByDependenciasLicitantes,
            },
        },
        {
            width: 200,
            name: "Dependencia usuario",
            filter: {
                appliedFilters: filterByDependenciasUsuarios,
                filterTitle: "Filtrar por dependencias usuarios",
                filters: filtroDependenciaUsuarios,
                setFilters: setFilterByDependenciasUsuarios,
            },
        },
        {
            width: 200,
            name: "Empresas participantes",
            filter: {
                appliedFilters: filterByEmpresasParticipantes,
                filterTitle: "Filtrar por empresas participantes",
                filters: filtroEmpresasParticipantes,
                setFilters: setFilterByEmpresasParticipantes,
            },
        },
        {
            width: 150,
            name: "Fecha publicación",
            filter: {
                appliedFilters: filterByFechasPublicacion,
                filterTitle: "Filtrar por fechas de publicación",
                filters: filtroFechasPublicacion,
                setFilters: setFilterByFechasPublicacion,
            },
        },
        {
            width: 250,
            name: "Dónde se publicó",
            filter: {
                appliedFilters: filterByLugaresPublicacion,
                filterTitle: "Filtrar por lugares de publicación",
                filters: filtroLugaresPublicacion,
                setFilters: setFilterByLugaresPublicacion,
            },
        },
        {
            width: 200,
            name: "Fecha de envío de preguntas",
            filter: {
                appliedFilters: filterByFechasEnvioPreguntas,
                filterTitle: "Filtrar por fechas de envío de preguntas",
                filters: filtroFechasPreguntas,
                setFilters: setFilterByFechasEnvioPreguntas,
            },
        },
        {
            width: 200,
            name: "Hora límite envío de preguntas",
            filter: {
                appliedFilters: filterByHorasLimiteEnviop,
                filterTitle: "Filtrar por hora límite envío de preguntas",
                filters: filtroHorasLimiteEnviop,
                setFilters: setFilterByHorasLimiteEnviop,
            },
        },
        {
            width: 150,
            name: "Estatus preguntas",
            filter: {
                appliedFilters: filterByPreguntas,
                filterTitle: "Filtrar por estatus preguntas",
                filters: filtroPreguntas,
                setFilters: setFilterByPreguntas,
            },
        },
        {
            width: 150,
            name: "Cómo se entregan",
            filter: {
                appliedFilters: filterByTiposDeEntrega,
                filterTitle: "Filtrar por tipo de entrega",
                filters: filtroTiposDeEntrega,
                setFilters: setFilterByTiposDeEntrega,
            },
        },
        {
            width: 350,
            name: "Domicilio y/o correo",
            filter: {
                appliedFilters: filterByDomiciliosCorreos,
                filterTitle: "Filtrar por domicilios y/o correos",
                filters: filtroDomiciliosCorreos,
                setFilters: setFilterByDomiciliosCorreos,
            },
        },
        {
            width: 150,
            name: "Fecha junta de aclaraciones",
            filter: {
                appliedFilters: filterByFechasJuntaAcl,
                filterTitle: "Filtrar por fechas de junta aclaratoria",
                filters: filtroFechasJuntaAcl,
                setFilters: setFilterByFechasJuntaAcl,
            },
        },
        {
            width: 150,
            name: "Hora registro JA",
            filter: {
                appliedFilters: filterByHorasRegistroJA,
                filterTitle:
                    "Filtrar por hora de registro de junta aclaratoria",
                filters: filtroHorasRegistroJA,
                setFilters: setFilterByHorasRegistroJA,
            },
        },
        {
            width: 100,
            name: "Hora JA",
            filter: {
                appliedFilters: filterByHorasJA,
                filterTitle: "Filtrar por horas de junta aclaratoria",
                filters: filtroHorasJA,
                setFilters: setFilterByHorasJA,
            },
        },
        {
            width: 100,
            name: "Estatus junta JA",
            filter: null,
        },
        {
            width: 150,
            name: "¿La junta es obligatoria?",
            filter: {
                appliedFilters: filterByJuntasObligatorias,
                filterTitle: "Filtrar por juntas obligatorias",
                filters: filtroJuntasObligatorias,
                setFilters: setFilterByJuntasObligatorias,
            },
        },
        {
            width: 350,
            name: "Domicilio JA",
            filter: {
                appliedFilters: filterByDomiciliosJunta,
                filterTitle: "Filtrar domicilios junta",
                filters: filtroDomiciliosJunta,
                setFilters: setFilterByDomiciliosJunta,
            },
        },
        {
            width: 150,
            name: "Es presencial o digital",
            filter: {
                appliedFilters: filterByTiposJuntas,
                filterTitle: "Filtrar tipo de junta",
                filters: filtroTiposJuntas,
                setFilters: setFilterByTiposJuntas,
            },
        },
        {
            width: 350,
            name: "Domicilio",
            filter: {
                appliedFilters: filterByDomicilios,
                filterTitle: "Filtrar por domicilio",
                filters: filtroDomicilios,
                setFilters: setFilterByDomicilios,
            },
        },
        {
            width: 100,
            name: "Productos",
            filter: null,
        },
        {
            width: 150,
            name: "Ficha técnica",
            filter: {
                appliedFilters: filterByEstatusFicha,
                filterTitle: "Filtrar por estatus ficha",
                filters: filtroEstatusFicha,
                setFilters: setFilterByEstatusFicha,
            },
        },
        {
            width: 150,
            name: "Fecha de presentación",
            filter: {
                appliedFilters: filterByFechasPre,
                filterTitle: "Filtrar por fechas de presentación",
                filters: filtroFechasPre,
                setFilters: setFilterByFechasPre,
            },
        },
        {
            width: 150,
            name: "Hora de registro para presentación",
            filter: {
                appliedFilters: filterByHorasRegistroPre,
                filterTitle: "Filtrar por horas de registro de presentación",
                filters: filtroHorasRegistroPre,
                setFilters: setFilterByHorasRegistroPre,
            },
        },
        {
            width: 100,
            name: "Hora de apertura",
            filter: {
                appliedFilters: filterByHorasApertura,
                filterTitle: "Filtrar por horas de apertura",
                filters: filtroHorasApertura,
                setFilters: setFilterByHorasApertura,
            },
        },
        {
            width: 150,
            name: "Estatus presentación",
            filter: {
                appliedFilters: filterByPresentacion,
                filterTitle: "Filtrar por estatus presentación",
                filters: filtroPresentacion,
                setFilters: setFilterByPresentacion,
            },
        },
        {
            width: 200,
            name: "Razón de no presentación",
            filter: null,
        },
        {
            width: 350,
            name: "Partidas",
            filter: {
                appliedFilters: filterByPartidas,
                filterTitle: "Filtrar por partidas",
                filters: filtroPartidas,
                setFilters: setFilterByPartidas,
            },
        },
        {
            width: 150,
            name: "Requiere preventa",
            filter: {
                appliedFilters: filterByRequierePreventa,
                filterTitle: "Filtrar por preventa requerida",
                filters: filtroRequierePreventa,
                setFilters: setFilterByRequierePreventa,
            },
        },
        {
            width: 350,
            name: "Link carpeta",
            filter: null,
        },
        {
            width: 150,
            name: "Estatus final",
            filter: {
                appliedFilters: filterByEstatusFinal,
                filterTitle: "Filtrar por estatus final",
                filters: estatusFinal,
                setFilters: setFilterByEstatusFinal,
            },
        },
        {
            width: 350,
            name: "Observaciones",
            filter: null,
        },
        {
            width: 150,
            name: "Fecha límite entrega de bienes",
            filter: null,
        },
    ];

    const scrollableRows = bids.map((bid) => {
        return {
            id: bid.id,
            rowClass: RowColor.get(bid.estatus_final),
            cells: [
                {
                    id: Math.random() * 100,
                    width: 150,
                    value: DateFormatter.fullDateNormal(bid.marca_temporal),
                },
                {
                    id: Math.random() * 100,
                    width: 250,
                    value: bid.usuario,
                },
                {
                    id: Math.random() * 100,
                    width: 250,
                    value: bid.vendedor ?? "-",
                    onClick: bid.vendedor
                        ? null
                        : () => {
                              setAssignationType("Vendedor");
                              setUserType("seller");
                              setRegistry(bid);
                              setUrlAssignUser(endpoints.bid.assign_seller);
                              setShowFrmAssignUser(true);
                          },
                },
                {
                    id: Math.random() * 100,
                    width: 250,
                    value: bid.seguidor ?? "-",
                    onClick: () => {
                        setAssignationType("Acompañante");
                        setUserType("follower");
                        setRegistry(bid);
                        setUrlAssignUser(endpoints.bid.assign_follower);
                        setShowFrmAssignUser(true);
                    },
                },
                {
                    id: Math.random() * 100,
                    width: 200,
                    value: bid.dependencia_licitante,
                },
                {
                    id: Math.random() * 100,
                    width: 200,
                    value: bid.dependencia_usuario,
                },
                {
                    id: Math.random() * 100,
                    width: 200,
                    value: bid.empresa_participantes,
                },
                {
                    id: Math.random() * 100,
                    width: 150,
                    value: DateFormatter.dateNormal(bid.fecha_publicacion),
                },
                {
                    id: Math.random() * 100,
                    width: 250,
                    value: bid.donde_se_publico,
                },
                {
                    id: Math.random() * 100,
                    width: 200,
                    value: DateFormatter.dateNormal(bid.fecha_envio_preguntas),
                },
                {
                    id: Math.random() * 100,
                    width: 200,
                    value: bid.hora_limite_envio_preguntas,
                },
                {
                    id: Math.random() * 100,
                    width: 150,
                    value: bid.estatus_preguntas,
                    onClick:
                        bid.estatus_final == "Se está trabajando" ||
                        bid.deshabilitado == 1
                            ? () => {
                                  setRegistry(bid);
                                  setUrlUpdate(endpoints.status.questions);
                                  setStatusType("preguntas");
                                  setShowFrmUpdateStatus(true);
                              }
                            : null,
                },
                {
                    id: Math.random() * 100,
                    width: 150,
                    value: bid.como_se_entregan,
                },
                {
                    id: Math.random() * 100,
                    width: 350,
                    value: bid.domicilio_correo,
                },
                {
                    id: Math.random() * 100,
                    width: 150,
                    value: DateFormatter.dateNormal(
                        bid.fecha_junta_aclaraciones
                    ),
                },
                {
                    id: Math.random() * 100,
                    width: 150,
                    value: bid.hora_registro_ja,
                },
                {
                    id: Math.random() * 100,
                    width: 100,
                    value: bid.hora_ja,
                },
                {
                    id: Math.random() * 100,
                    width: 100,
                    value: bid.estatus_junta,
                    onClick:
                        bid.estatus_final == "Se está trabajando" ||
                        bid.deshabilitado == 1
                            ? () => {
                                  setRegistry(bid);
                                  setUrlUpdate(endpoints.status.meeting);
                                  setStatusType("junta");
                                  setShowFrmUpdateStatus(true);
                              }
                            : null,
                },
                {
                    id: Math.random() * 100,
                    width: 150,
                    value: bid.junta_obligatoria,
                },
                {
                    id: Math.random() * 100,
                    width: 350,
                    value: bid.domicilio_ja,
                },
                {
                    id: Math.random() * 100,
                    width: 150,
                    value: bid.presencial_digital,
                },
                {
                    id: Math.random() * 100,
                    width: 350,
                    value: bid.domicilio,
                },
                {
                    id: Math.random() * 100,
                    width: 100,
                    value: bid.estatus_productos ?? "",
                    onClick: () => {
                        setRegistry(bid);
                        setShowFrmProductStatus(true);
                    },
                },
                {
                    id: Math.random() * 100,
                    width: 150,
                    value: bid.estatus_ficha,
                    onClick:
                        bid.estatus_final == "Se está trabajando" ||
                        bid.deshabilitado == 1
                            ? () => {
                                  setRegistry(bid);
                                  setUrlUpdate(endpoints.status.ticket);
                                  setStatusType("ficha");
                                  setShowFrmUpdateStatus(true);
                              }
                            : null,
                },
                {
                    id: Math.random() * 100,
                    width: 150,
                    value: DateFormatter.dateNormal(bid.fecha_presentacion),
                },
                {
                    id: Math.random() * 100,
                    width: 150,
                    value: bid.hora_registro_presentacion,
                },
                {
                    id: Math.random() * 100,
                    width: 100,
                    value: bid.hora_apertura,
                },
                {
                    id: Math.random() * 100,
                    width: 150,
                    value: bid.estatus_presentacion,
                    onClick:
                        bid.estatus_final == "Se está trabajando" ||
                        bid.deshabilitado == 1
                            ? () => {
                                  setRegistry(bid);
                                  setUrlUpdate(endpoints.status.presentation);
                                  setStatusType("presentacion");
                                  setShowFrmUpdateStatus(true);
                              }
                            : null,
                },
                {
                    id: Math.random() * 100,
                    width: 200,
                    value: bid.razon_no_presentacion,
                },
                {
                    id: Math.random() * 100,
                    width: 350,
                    value: bid.partidas,
                },
                {
                    id: Math.random() * 100,
                    width: 150,
                    value: bid.requiere_preventa,
                },
                {
                    id: Math.random() * 100,
                    width: 350,
                    value: bid.link_carpeta,
                    link: true,
                },
                {
                    id: Math.random() * 100,
                    width: 150,
                    value: bid.estatus_final,
                    onClick:
                        bid.estatus_final != null && bid.estatus_final != ""
                            ? () => {
                                  setRegistry(bid);
                                  setUrlUpdate(endpoints.status.final);
                                  setStatusType("final");
                                  setShowFrmUpdateStatus(true);
                              }
                            : null,
                },
                {
                    id: Math.random() * 100,
                    width: 350,
                    value: bid.observaciones,
                    onClick: () => {
                        setRegistry(bid);
                        setShowFrmRemark(true);
                    },
                },
                {
                    id: Math.random() * 100,
                    width: 150,
                    value: bid.fecha_limite_entregab
                        ? DateFormatter.date(bid.fecha_limite_entregab, true)
                        : "Establecer",
                    onClick:
                        bid.estatus_final == "Se ganó"
                            ? () => {
                                  setRegistry(bid);
                                  setShowFrmUpdateDate(true);
                              }
                            : null,
                },
            ],
        };
    });

    return (
        <section className='Page__Container'>
            <Head>
                <title>{process.env.REACT_APP_TITLE + " - Licitaciones"}</title>
            </Head>
            <div className='Page__Options'>
                <div
                    className='Option'
                    onClick={() => setShowFrmNotification(true)}>
                    Notificaciones
                </div>
                <div className='Option' onClick={() => setShowLstHistory(true)}>
                    Historial
                </div>
                <div className='Option' onClick={() => setShowCharts(true)}>
                    Gráfica
                </div>
                <div
                    className='Option'
                    onClick={() => {
                        setRegistry(null);
                        setShowFrmBid(true);
                    }}>
                    Formulario
                </div>

                {/* OPTIONS MODALS */}
                {showFrmNotification && (
                    <FrmNotification
                        title={"Registrar notificación"}
                        url={endpoints.email_notification.store}
                        telegramLink={"https://t.me/+wsy-EWMLw4o1MTNh"}
                        closeModal={() => setShowFrmNotification(false)}
                    />
                )}

                {showLstHistory && (
                    <LstHistory
                        registries={bids}
                        closeModal={() => setShowLstHistory(false)}
                    />
                )}

                {showCharts && (
                    <ChartsContainer
                        title={"Estadísticas de las licitaciones"}
                        limit={totalItems}
                        url={endpoints.bid.get}
                        years={filtroAnios}
                        label={"Licitaciones"}
                        closeModal={() => setShowCharts(false)}
                    />
                )}

                {showFrmBid && (
                    <FrmBid
                        registry={registry}
                        closeModal={() => setShowFrmBid(false)}
                        updateData={updateData}
                        url={url}
                    />
                )}
            </div>
            <div className='Page__Wrapper'>
                {bids.length == 0 && (
                    <NoData message={"Sin registro de licitaciones"} />
                )}
                <div className='Content'>
                    <div className='Sticky__Container'>
                        <Table
                            onscroll={() =>
                                Scroll.onscroll(
                                    `.Page__Wrapper .Content .Sticky__Container .Table .Body`,
                                    `.Page__Wrapper .Content .Tbl__Scrollable .Table .Body`
                                )
                            }
                            columns={stikyColumns}
                            rows={stickyRows}
                        />
                    </div>
                    <div className='Tbl__Scrollable'>
                        <Table
                            onscroll={() =>
                                Scroll.onscroll(
                                    `.Page__Wrapper .Content .Tbl__Scrollable .Table .Body`,
                                    `.Page__Wrapper .Content .Sticky__Container .Table .Body`
                                )
                            }
                            sticky={true}
                            columns={scrollableColumns}
                            rows={scrollableRows}
                        />
                    </div>

                    {/* REGISTRY MODALS */}

                    {showRegistryHistory && (
                        <RegistryHistoryDetail
                            registry={registry}
                            closeModal={() => setShowRegistryHistory(false)}
                        />
                    )}

                    {showFrmUpdateStatus && (
                        <FrmUpdateStatus
                            registry={registry}
                            url={urlUpdate}
                            closeModal={() => setShowFrmUpdateStatus(false)}
                            urlData={url}
                            updateData={updateData}
                            type={statusType}
                            updateFinalEstatus={updateFinalEstatus}
                        />
                    )}

                    {showFrmUpdateDate && (
                        <FrmUpdateDate
                            registry={registry}
                            closeModal={() => setShowFrmUpdateDate(false)}
                            updateData={updateData}
                            url={url}
                        />
                    )}

                    {showFrmRemark && (
                        <FrmRemark
                            registry={registry}
                            closeModal={() => setShowFrmRemark(false)}
                            url={endpoints.status.remarks}
                            urlData={url}
                            updateData={updateData}
                        />
                    )}

                    {showFrmAssignUser && (
                        <FrmAssignUser
                            type={assignationType}
                            url={urlAssignUser}
                            registry={registry}
                            closeModal={() => setShowFrmAssignUser(false)}
                            urlData={url}
                            updateData={updateData}
                            entityType={"bid"}
                            userType={userType}
                            telegramChannel={TelegramChannels.bidChannel}
                        />
                    )}

                    {showFrmProductStatus ? (
                        <FrmProductSatus
                            registry={registry}
                            urlData={url}
                            updateData={updateData}
                            closeModal={() => setShowFrmProductStatus(false)}
                        />
                    ) : null}
                </div>

                <div className='Paginator'>
                    <BidLegend />

                    {/* FILTERS BY DATE */}
                    <div className='DateFilters'>
                        <div className='Filter'>
                            <label htmlFor='month'>Filtrar por mes</label>
                            <select
                                name='month'
                                id='month'
                                onChange={() => onChangeSelection()}>
                                {filtroMes.map((option) => (
                                    <option value={option.no} key={option.no}>
                                        {option.mes}
                                    </option>
                                ))}
                            </select>
                        </div>

                        <div className='Filter'>
                            <label htmlFor='year'>Año</label>
                            <select
                                name='year'
                                id='year'
                                onChange={() => onChangeSelection()}>
                                {filtroAnios.map((anio) => (
                                    <option value={anio} key={anio}>
                                        {anio}
                                    </option>
                                ))}
                            </select>
                        </div>
                    </div>

                    {/* PAGINATOR CONTROLS */}
                    <div className='Paginator__Controls'>
                        <div className='Paginator__CurrentPage'>
                            <label htmlFor='currentPage'>Página</label>
                            <input
                                type='text'
                                name='currentPage'
                                id='currentPage'
                                defaultValue={currPage}
                                onChange={(e) => onChangePage(e)}
                            />
                        </div>

                        <div className='Paginator__RecordsShown'>
                            <label htmlFor='perPage'>
                                Registros por página
                            </label>
                            <input
                                type='text'
                                name='perPage'
                                id='perPage'
                                defaultValue={perPage}
                                onChange={(e) => onChangeItemsPerPage(e)}
                            />
                        </div>

                        <div className='Paginator__SomeOfLabels'>
                            {itemsOf}-{itemsTo} de {itemsCount}
                        </div>

                        <div className='Paginator__Buttons'>
                            <span
                                className={`Prev__Button ${
                                    prevPage ? "" : "disabled"
                                }`}
                                onClick={
                                    prevPage ? () => getPage(prevPage) : null
                                }>
                                {"<"}
                            </span>
                            <span
                                className={`Next__Button ${
                                    nextPage ? "" : "disabled"
                                }`}
                                onClick={
                                    nextPage ? () => getPage(nextPage) : null
                                }>
                                {">"}
                            </span>
                        </div>
                    </div>
                </div>
            </div>
        </section>
    );
};

export default Bid;
